//line parser/parser.y:2
package parser

import __yyfmt__ "fmt"

//line parser/parser.y:2
import (
	"fmt"
	"github.com/z7zmey/php-parser/node"
	"github.com/z7zmey/php-parser/node/expr"
	"github.com/z7zmey/php-parser/node/expr/assign_op"
	"github.com/z7zmey/php-parser/node/expr/binary_op"
	"github.com/z7zmey/php-parser/node/expr/cast"
	"github.com/z7zmey/php-parser/node/name"
	"github.com/z7zmey/php-parser/node/scalar"
	"github.com/z7zmey/php-parser/node/stmt"
	"github.com/z7zmey/php-parser/token"
	"io"
	"strconv"
	"strings"
)

var rootnode = stmt.NewStmtList([]node.Node{})

func Parse(src io.Reader, fName string) node.Node {
	yyDebug = 0
	yyErrorVerbose = true
	rootnode = stmt.NewStmtList([]node.Node{}) //reset
	yyParse(newLexer(src, fName))
	return rootnode
}

type foreachVariable struct {
	node  node.Node
	byRef bool
}

//line parser/parser.y:37
type yySymType struct {
	yys             int
	node            node.Node
	token           token.Token
	boolean         bool
	list            []node.Node
	foreachVariable foreachVariable
}

const T_INCLUDE = 57346
const T_INCLUDE_ONCE = 57347
const T_EVAL = 57348
const T_REQUIRE = 57349
const T_REQUIRE_ONCE = 57350
const T_LOGICAL_OR = 57351
const T_LOGICAL_XOR = 57352
const T_LOGICAL_AND = 57353
const T_PRINT = 57354
const T_YIELD = 57355
const T_DOUBLE_ARROW = 57356
const T_YIELD_FROM = 57357
const T_PLUS_EQUAL = 57358
const T_MINUS_EQUAL = 57359
const T_MUL_EQUAL = 57360
const T_DIV_EQUAL = 57361
const T_CONCAT_EQUAL = 57362
const T_MOD_EQUAL = 57363
const T_AND_EQUAL = 57364
const T_OR_EQUAL = 57365
const T_XOR_EQUAL = 57366
const T_SL_EQUAL = 57367
const T_SR_EQUAL = 57368
const T_POW_EQUAL = 57369
const T_COALESCE = 57370
const T_BOOLEAN_OR = 57371
const T_BOOLEAN_AND = 57372
const T_IS_EQUAL = 57373
const T_IS_NOT_EQUAL = 57374
const T_IS_IDENTICAL = 57375
const T_IS_NOT_IDENTICAL = 57376
const T_SPACESHIP = 57377
const T_IS_SMALLER_OR_EQUAL = 57378
const T_IS_GREATER_OR_EQUAL = 57379
const T_SL = 57380
const T_SR = 57381
const T_INSTANCEOF = 57382
const T_INC = 57383
const T_DEC = 57384
const T_INT_CAST = 57385
const T_DOUBLE_CAST = 57386
const T_STRING_CAST = 57387
const T_ARRAY_CAST = 57388
const T_OBJECT_CAST = 57389
const T_BOOL_CAST = 57390
const T_UNSET_CAST = 57391
const T_POW = 57392
const T_NEW = 57393
const T_CLONE = 57394
const T_NOELSE = 57395
const T_ELSEIF = 57396
const T_ELSE = 57397
const T_ENDIF = 57398
const T_STATIC = 57399
const T_ABSTRACT = 57400
const T_FINAL = 57401
const T_PRIVATE = 57402
const T_PROTECTED = 57403
const T_PUBLIC = 57404
const T_EXIT = 57405
const T_IF = 57406
const T_LNUMBER = 57407
const T_DNUMBER = 57408
const T_STRING = 57409
const T_STRING_VARNAME = 57410
const T_VARIABLE = 57411
const T_NUM_STRING = 57412
const T_INLINE_HTML = 57413
const T_CHARACTER = 57414
const T_BAD_CHARACTER = 57415
const T_ENCAPSED_AND_WHITESPACE = 57416
const T_CONSTANT_ENCAPSED_STRING = 57417
const T_ECHO = 57418
const T_DO = 57419
const T_WHILE = 57420
const T_ENDWHILE = 57421
const T_FOR = 57422
const T_ENDFOR = 57423
const T_FOREACH = 57424
const T_ENDFOREACH = 57425
const T_DECLARE = 57426
const T_ENDDECLARE = 57427
const T_AS = 57428
const T_SWITCH = 57429
const T_ENDSWITCH = 57430
const T_CASE = 57431
const T_DEFAULT = 57432
const T_BREAK = 57433
const T_CONTINUE = 57434
const T_GOTO = 57435
const T_FUNCTION = 57436
const T_CONST = 57437
const T_RETURN = 57438
const T_TRY = 57439
const T_CATCH = 57440
const T_FINALLY = 57441
const T_THROW = 57442
const T_USE = 57443
const T_INSTEADOF = 57444
const T_GLOBAL = 57445
const T_VAR = 57446
const T_UNSET = 57447
const T_ISSET = 57448
const T_EMPTY = 57449
const T_HALT_COMPILER = 57450
const T_CLASS = 57451
const T_TRAIT = 57452
const T_INTERFACE = 57453
const T_EXTENDS = 57454
const T_IMPLEMENTS = 57455
const T_OBJECT_OPERATOR = 57456
const T_LIST = 57457
const T_ARRAY = 57458
const T_CALLABLE = 57459
const T_CLASS_C = 57460
const T_TRAIT_C = 57461
const T_METHOD_C = 57462
const T_FUNC_C = 57463
const T_LINE = 57464
const T_FILE = 57465
const T_COMMENT = 57466
const T_DOC_COMMENT = 57467
const T_OPEN_TAG = 57468
const T_OPEN_TAG_WITH_ECHO = 57469
const T_CLOSE_TAG = 57470
const T_WHITESPACE = 57471
const T_START_HEREDOC = 57472
const T_END_HEREDOC = 57473
const T_DOLLAR_OPEN_CURLY_BRACES = 57474
const T_CURLY_OPEN = 57475
const T_PAAMAYIM_NEKUDOTAYIM = 57476
const T_NAMESPACE = 57477
const T_NS_C = 57478
const T_DIR = 57479
const T_NS_SEPARATOR = 57480
const T_ELLIPSIS = 57481

var yyToknames = [...]string{
	"$end",
	"error",
	"$unk",
	"T_INCLUDE",
	"T_INCLUDE_ONCE",
	"T_EVAL",
	"T_REQUIRE",
	"T_REQUIRE_ONCE",
	"','",
	"T_LOGICAL_OR",
	"T_LOGICAL_XOR",
	"T_LOGICAL_AND",
	"T_PRINT",
	"T_YIELD",
	"T_DOUBLE_ARROW",
	"T_YIELD_FROM",
	"'='",
	"T_PLUS_EQUAL",
	"T_MINUS_EQUAL",
	"T_MUL_EQUAL",
	"T_DIV_EQUAL",
	"T_CONCAT_EQUAL",
	"T_MOD_EQUAL",
	"T_AND_EQUAL",
	"T_OR_EQUAL",
	"T_XOR_EQUAL",
	"T_SL_EQUAL",
	"T_SR_EQUAL",
	"T_POW_EQUAL",
	"'?'",
	"':'",
	"T_COALESCE",
	"T_BOOLEAN_OR",
	"T_BOOLEAN_AND",
	"'|'",
	"'^'",
	"'&'",
	"T_IS_EQUAL",
	"T_IS_NOT_EQUAL",
	"T_IS_IDENTICAL",
	"T_IS_NOT_IDENTICAL",
	"T_SPACESHIP",
	"'<'",
	"T_IS_SMALLER_OR_EQUAL",
	"'>'",
	"T_IS_GREATER_OR_EQUAL",
	"T_SL",
	"T_SR",
	"'+'",
	"'-'",
	"'.'",
	"'*'",
	"'/'",
	"'%'",
	"'!'",
	"T_INSTANCEOF",
	"'~'",
	"T_INC",
	"T_DEC",
	"T_INT_CAST",
	"T_DOUBLE_CAST",
	"T_STRING_CAST",
	"T_ARRAY_CAST",
	"T_OBJECT_CAST",
	"T_BOOL_CAST",
	"T_UNSET_CAST",
	"'@'",
	"T_POW",
	"'['",
	"T_NEW",
	"T_CLONE",
	"T_NOELSE",
	"T_ELSEIF",
	"T_ELSE",
	"T_ENDIF",
	"T_STATIC",
	"T_ABSTRACT",
	"T_FINAL",
	"T_PRIVATE",
	"T_PROTECTED",
	"T_PUBLIC",
	"T_EXIT",
	"T_IF",
	"T_LNUMBER",
	"T_DNUMBER",
	"T_STRING",
	"T_STRING_VARNAME",
	"T_VARIABLE",
	"T_NUM_STRING",
	"T_INLINE_HTML",
	"T_CHARACTER",
	"T_BAD_CHARACTER",
	"T_ENCAPSED_AND_WHITESPACE",
	"T_CONSTANT_ENCAPSED_STRING",
	"T_ECHO",
	"T_DO",
	"T_WHILE",
	"T_ENDWHILE",
	"T_FOR",
	"T_ENDFOR",
	"T_FOREACH",
	"T_ENDFOREACH",
	"T_DECLARE",
	"T_ENDDECLARE",
	"T_AS",
	"T_SWITCH",
	"T_ENDSWITCH",
	"T_CASE",
	"T_DEFAULT",
	"T_BREAK",
	"T_CONTINUE",
	"T_GOTO",
	"T_FUNCTION",
	"T_CONST",
	"T_RETURN",
	"T_TRY",
	"T_CATCH",
	"T_FINALLY",
	"T_THROW",
	"T_USE",
	"T_INSTEADOF",
	"T_GLOBAL",
	"T_VAR",
	"T_UNSET",
	"T_ISSET",
	"T_EMPTY",
	"T_HALT_COMPILER",
	"T_CLASS",
	"T_TRAIT",
	"T_INTERFACE",
	"T_EXTENDS",
	"T_IMPLEMENTS",
	"T_OBJECT_OPERATOR",
	"T_LIST",
	"T_ARRAY",
	"T_CALLABLE",
	"T_CLASS_C",
	"T_TRAIT_C",
	"T_METHOD_C",
	"T_FUNC_C",
	"T_LINE",
	"T_FILE",
	"T_COMMENT",
	"T_DOC_COMMENT",
	"T_OPEN_TAG",
	"T_OPEN_TAG_WITH_ECHO",
	"T_CLOSE_TAG",
	"T_WHITESPACE",
	"T_START_HEREDOC",
	"T_END_HEREDOC",
	"T_DOLLAR_OPEN_CURLY_BRACES",
	"T_CURLY_OPEN",
	"T_PAAMAYIM_NEKUDOTAYIM",
	"T_NAMESPACE",
	"T_NS_C",
	"T_DIR",
	"T_NS_SEPARATOR",
	"T_ELLIPSIS",
	"'\"'",
	"'`'",
	"'{'",
	"'}'",
	"';'",
	"'('",
	"')'",
	"']'",
	"'$'",
}
var yyStatenames = [...]string{}

const yyEofCode = 1
const yyErrCode = 2
const yyInitialStackSize = 16

//line parser/parser.y:1172

//line yacctab:1
var yyExca = [...]int{
	-1, 1,
	1, -1,
	-2, 0,
	-1, 43,
	69, 419,
	133, 419,
	153, 419,
	161, 419,
	-2, 414,
	-1, 47,
	164, 422,
	-2, 431,
	-1, 83,
	69, 421,
	133, 421,
	153, 421,
	161, 421,
	164, 424,
	-2, 409,
	-1, 106,
	153, 382,
	-2, 411,
	-1, 228,
	69, 419,
	133, 419,
	153, 419,
	161, 419,
	-2, 311,
	-1, 231,
	164, 424,
	-2, 421,
	-1, 234,
	69, 419,
	133, 419,
	153, 419,
	161, 419,
	-2, 313,
	-1, 352,
	38, 0,
	39, 0,
	40, 0,
	41, 0,
	42, 0,
	-2, 335,
	-1, 353,
	38, 0,
	39, 0,
	40, 0,
	41, 0,
	42, 0,
	-2, 336,
	-1, 354,
	38, 0,
	39, 0,
	40, 0,
	41, 0,
	42, 0,
	-2, 337,
	-1, 355,
	38, 0,
	39, 0,
	40, 0,
	41, 0,
	42, 0,
	-2, 338,
	-1, 356,
	43, 0,
	44, 0,
	45, 0,
	46, 0,
	-2, 339,
	-1, 357,
	43, 0,
	44, 0,
	45, 0,
	46, 0,
	-2, 340,
	-1, 358,
	43, 0,
	44, 0,
	45, 0,
	46, 0,
	-2, 341,
	-1, 359,
	43, 0,
	44, 0,
	45, 0,
	46, 0,
	-2, 342,
	-1, 360,
	38, 0,
	39, 0,
	40, 0,
	41, 0,
	42, 0,
	-2, 343,
	-1, 367,
	9, 161,
	165, 161,
	-2, 419,
	-1, 374,
	165, 211,
	-2, 216,
	-1, 411,
	9, 459,
	165, 459,
	166, 459,
	-2, 419,
	-1, 415,
	69, 420,
	133, 420,
	153, 420,
	161, 420,
	164, 423,
	-2, 345,
	-1, 429,
	164, 445,
	-2, 412,
	-1, 430,
	164, 447,
	-2, 437,
	-1, 509,
	164, 445,
	-2, 413,
	-1, 510,
	164, 447,
	-2, 438,
	-1, 558,
	165, 211,
	-2, 216,
	-1, 568,
	165, 211,
	-2, 216,
	-1, 603,
	164, 423,
	-2, 420,
	-1, 659,
	165, 181,
	-2, 419,
	-1, 681,
	88, 263,
	-2, 266,
	-1, 700,
	9, 458,
	165, 458,
	166, 458,
	-2, 419,
	-1, 733,
	165, 182,
	-2, 419,
	-1, 785,
	165, 181,
	-2, 419,
	-1, 787,
	165, 184,
	-2, 393,
	-1, 858,
	105, 76,
	-2, 80,
	-1, 881,
	165, 211,
	-2, 216,
}

const yyPrivate = 57344

const yyLast = 7406

var yyAct = [...]int{

	27, 129, 791, 803, 585, 683, 434, 569, 837, 850,
	802, 781, 819, 749, 106, 125, 573, 658, 741, 583,
	640, 137, 137, 137, 725, 714, 150, 311, 110, 639,
	4, 320, 520, 571, 185, 530, 366, 563, 580, 114,
	120, 574, 222, 403, 220, 131, 188, 79, 149, 376,
	146, 315, 224, 227, 8, 83, 235, 236, 237, 238,
	239, 43, 136, 240, 241, 242, 243, 244, 245, 246,
	314, 249, 142, 7, 257, 258, 259, 313, 127, 312,
	6, 428, 5, 512, 139, 140, 511, 787, 272, 273,
	126, 275, 276, 707, 124, 703, 617, 253, 263, 268,
	604, 229, 229, 2, 592, 408, 876, 232, 104, 231,
	231, 81, 307, 907, 893, 228, 234, 116, 828, 104,
	827, 104, 817, 809, 740, 731, 729, 712, 710, 702,
	668, 317, 656, 644, 635, 322, 323, 305, 605, 691,
	289, 596, 291, 290, 373, 288, 915, 298, 914, 293,
	304, 292, 820, 335, 336, 337, 338, 339, 340, 341,
	342, 343, 344, 345, 346, 347, 348, 349, 350, 351,
	352, 353, 354, 355, 356, 357, 358, 359, 360, 319,
	362, 364, 404, 368, 104, 133, 370, 105, 111, 881,
	284, 286, 331, 268, 282, 909, 265, 78, 105, 845,
	105, 386, 388, 389, 390, 391, 392, 393, 394, 395,
	396, 397, 398, 399, 799, 794, 400, 137, 402, 294,
	224, 143, 374, 789, 382, 734, 775, 776, 701, 229,
	285, 413, 568, 558, 224, 775, 776, 231, 374, 552,
	550, 412, 383, 367, 301, 381, 334, 287, 232, 137,
	687, 688, 689, 686, 685, 684, 423, 405, 116, 274,
	104, 137, 875, 105, 407, 429, 509, 514, 566, 271,
	330, 521, 522, 229, 369, 523, 328, 361, 414, 401,
	822, 231, 269, 527, 821, 307, 531, 411, 224, 774,
	265, 807, 270, 806, 680, 248, 219, 682, 215, 301,
	267, 183, 182, 283, 229, 181, 660, 135, 134, 549,
	130, 416, 231, 112, 892, 305, 885, 884, 425, 866,
	829, 823, 4, 427, 816, 772, 133, 759, 541, 111,
	544, 516, 533, 559, 730, 150, 859, 116, 662, 105,
	728, 726, 724, 721, 508, 232, 8, 614, 553, 418,
	419, 517, 538, 844, 518, 116, 535, 104, 384, 515,
	372, 653, 326, 109, 562, 7, 325, 324, 798, 295,
	860, 216, 6, 815, 5, 418, 269, 419, 419, 418,
	812, 557, 587, 560, 588, 810, 548, 589, 590, 578,
	768, 615, 582, 906, 867, 586, 542, 536, 539, 835,
	333, 556, 833, 661, 108, 790, 595, 764, 113, 746,
	224, 599, 115, 224, 674, 534, 406, 584, 534, 378,
	184, 745, 417, 133, 329, 579, 111, 616, 113, 288,
	327, 581, 619, 230, 534, 218, 105, 116, 534, 306,
	613, 888, 154, 156, 155, 687, 688, 689, 686, 685,
	684, 116, 598, 300, 513, 377, 254, 602, 303, 430,
	510, 420, 179, 217, 180, 152, 153, 157, 159, 158,
	171, 172, 169, 170, 177, 173, 174, 175, 176, 167,
	168, 161, 162, 160, 163, 165, 166, 594, 178, 680,
	716, 652, 682, 643, 687, 688, 689, 686, 685, 684,
	164, 687, 688, 689, 686, 685, 684, 116, 543, 637,
	288, 303, 618, 545, 262, 137, 623, 261, 303, 255,
	256, 380, 299, 886, 742, 421, 143, 526, 738, 739,
	145, 841, 116, 634, 122, 123, 887, 116, 680, 303,
	147, 682, 318, 303, 704, 680, 871, 254, 682, 119,
	752, 751, 420, 648, 322, 650, 877, 627, 750, 305,
	624, 641, 655, 638, 4, 145, 654, 632, 626, 575,
	547, 795, 544, 578, 544, 147, 667, 622, 121, 646,
	797, 576, 577, 578, 116, 649, 675, 755, 8, 422,
	578, 540, 620, 611, 664, 302, 586, 723, 699, 870,
	133, 657, 128, 111, 690, 133, 609, 7, 111, 525,
	255, 256, 229, 229, 6, 705, 5, 672, 673, 695,
	231, 231, 824, 775, 776, 116, 367, 659, 254, 521,
	778, 775, 776, 277, 531, 687, 688, 689, 686, 685,
	684, 254, 607, 546, 610, 608, 420, 534, 229, 722,
	76, 77, 375, 715, 194, 254, 231, 193, 719, 641,
	192, 711, 700, 224, 252, 709, 544, 736, 297, 713,
	186, 544, 544, 720, 576, 577, 839, 717, 727, 680,
	195, 196, 682, 260, 578, 753, 164, 754, 189, 671,
	278, 255, 256, 133, 747, 586, 111, 792, 760, 761,
	254, 190, 224, 743, 255, 256, 756, 735, 229, 116,
	758, 621, 178, 385, 406, 625, 231, 524, 255, 256,
	757, 676, 733, 254, 164, 322, 254, 838, 281, 187,
	698, 251, 715, 846, 641, 224, 122, 123, 840, 769,
	45, 800, 767, 597, 793, 544, 762, 544, 780, 332,
	784, 777, 765, 779, 308, 732, 773, 691, 770, 782,
	808, 766, 718, 255, 256, 642, 796, 279, 280, 687,
	688, 689, 686, 685, 684, 669, 818, 118, 191, 786,
	229, 633, 813, 825, 811, 826, 255, 256, 231, 255,
	256, 628, 832, 564, 785, 544, 197, 199, 198, 804,
	554, 843, 409, 1, 831, 814, 37, 578, 847, 834,
	681, 856, 678, 842, 679, 379, 913, 570, 529, 221,
	519, 865, 855, 167, 168, 161, 162, 160, 163, 165,
	166, 864, 178, 836, 365, 873, 805, 874, 849, 869,
	651, 878, 144, 748, 164, 862, 141, 880, 321, 148,
	801, 666, 830, 856, 250, 782, 879, 223, 42, 883,
	41, 15, 14, 606, 855, 889, 890, 266, 48, 891,
	87, 88, 89, 90, 91, 47, 107, 49, 82, 73,
	74, 80, 75, 71, 247, 897, 894, 61, 264, 898,
	60, 899, 903, 902, 901, 854, 578, 853, 163, 165,
	166, 586, 178, 852, 851, 677, 908, 44, 910, 572,
	911, 737, 663, 861, 164, 55, 56, 916, 645, 309,
	117, 57, 296, 58, 53, 54, 62, 63, 64, 65,
	66, 67, 68, 70, 3, 51, 84, 52, 433, 744,
	670, 0, 24, 76, 77, 0, 0, 0, 69, 46,
	92, 93, 35, 0, 104, 0, 26, 0, 0, 0,
	109, 25, 17, 16, 0, 18, 0, 29, 0, 30,
	0, 0, 19, 0, 0, 0, 20, 21, 34, 36,
	12, 22, 32, 0, 0, 33, 11, 0, 23, 0,
	28, 85, 86, 9, 38, 39, 40, 0, 0, 0,
	50, 108, 0, 101, 97, 98, 99, 94, 95, 0,
	0, 0, 0, 0, 0, 102, 0, 0, 0, 0,
	10, 100, 96, 111, 0, 103, 72, 13, 636, 31,
	59, 0, 0, 105, 87, 88, 89, 90, 91, 0,
	0, 0, 0, 73, 74, 179, 75, 180, 152, 153,
	157, 159, 158, 171, 172, 169, 170, 177, 173, 174,
	175, 176, 167, 168, 161, 162, 160, 163, 165, 166,
	0, 178, 0, 0, 0, 0, 0, 0, 0, 55,
	56, 0, 0, 164, 0, 57, 0, 58, 53, 54,
	62, 63, 64, 65, 66, 67, 68, 70, 0, 51,
	84, 52, 0, 0, 0, 0, 24, 76, 77, 0,
	0, 0, 69, 46, 92, 93, 35, 0, 104, 0,
	26, 0, 0, 0, 109, 25, 17, 16, 0, 18,
	0, 29, 0, 30, 0, 0, 19, 0, 0, 0,
	20, 21, 34, 36, 12, 22, 32, 0, 0, 33,
	11, 0, 23, 0, 28, 85, 86, 9, 38, 39,
	40, 0, 0, 0, 50, 108, 0, 101, 97, 98,
	99, 94, 95, 0, 0, 0, 0, 0, 0, 102,
	0, 0, 0, 0, 10, 100, 96, 111, 0, 103,
	72, 13, 537, 31, 59, 0, 0, 105, 87, 88,
	89, 90, 91, 0, 0, 0, 0, 73, 74, 0,
	75, 180, 152, 153, 157, 159, 158, 171, 172, 169,
	170, 177, 173, 174, 175, 176, 167, 168, 161, 162,
	160, 163, 165, 166, 0, 178, 0, 0, 0, 0,
	0, 0, 0, 55, 56, 0, 0, 164, 0, 57,
	0, 58, 53, 54, 62, 63, 64, 65, 66, 67,
	68, 70, 0, 51, 84, 52, 0, 0, 0, 0,
	24, 76, 77, 0, 0, 0, 69, 46, 92, 93,
	35, 0, 104, 0, 26, 0, 0, 0, 109, 25,
	17, 16, 0, 18, 0, 29, 0, 30, 0, 0,
	19, 0, 0, 0, 20, 21, 34, 36, 12, 22,
	32, 0, 0, 33, 11, 0, 23, 0, 28, 85,
	86, 9, 38, 39, 40, 0, 0, 0, 50, 108,
	0, 101, 97, 98, 99, 94, 95, 161, 162, 160,
	163, 165, 166, 102, 178, 0, 0, 0, 10, 100,
	96, 111, 0, 103, 72, 13, 164, 31, 59, 0,
	0, 105, 87, 88, 89, 90, 91, 0, 0, 0,
	0, 73, 74, 0, 75, 153, 157, 159, 158, 171,
	172, 169, 170, 177, 173, 174, 175, 176, 167, 168,
	161, 162, 160, 163, 165, 166, 0, 178, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 55, 56, 164,
	0, 0, 0, 57, 0, 58, 53, 54, 62, 63,
	64, 65, 66, 67, 68, 70, 0, 51, 84, 52,
	0, 0, 0, 0, 24, 76, 77, 0, 0, 0,
	69, 46, 92, 93, 35, 0, 104, 0, 26, 0,
	0, 0, 109, 25, 17, 16, 0, 18, 0, 29,
	0, 30, 0, 0, 19, 0, 0, 0, 20, 21,
	34, 36, 0, 22, 32, 0, 0, 33, 0, 0,
	23, 0, 28, 85, 86, 316, 38, 39, 40, 0,
	0, 0, 50, 108, 0, 101, 97, 98, 99, 94,
	95, 0, 0, 0, 0, 0, 0, 102, 0, 0,
	0, 0, 133, 100, 96, 111, 0, 103, 72, 13,
	917, 31, 59, 0, 0, 105, 87, 88, 89, 90,
	91, 0, 0, 0, 0, 73, 74, 0, 75, 0,
	157, 159, 158, 171, 172, 169, 170, 177, 173, 174,
	175, 176, 167, 168, 161, 162, 160, 163, 165, 166,
	0, 178, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 55, 56, 164, 0, 0, 0, 57, 0, 58,
	53, 54, 62, 63, 64, 65, 66, 67, 68, 70,
	0, 51, 84, 52, 0, 0, 0, 0, 24, 76,
	77, 0, 0, 0, 69, 46, 92, 93, 35, 0,
	104, 0, 26, 0, 0, 0, 109, 25, 17, 16,
	0, 18, 0, 29, 0, 30, 0, 0, 19, 0,
	0, 0, 20, 21, 34, 36, 0, 22, 32, 0,
	0, 33, 0, 0, 23, 0, 28, 85, 86, 316,
	38, 39, 40, 0, 0, 0, 50, 108, 0, 101,
	97, 98, 99, 94, 95, 0, 0, 0, 0, 0,
	0, 102, 0, 0, 0, 0, 133, 100, 96, 111,
	0, 103, 72, 13, 912, 31, 59, 0, 0, 105,
	87, 88, 89, 90, 91, 0, 0, 0, 0, 73,
	74, 0, 75, 0, 0, 159, 158, 171, 172, 169,
	170, 177, 173, 174, 175, 176, 167, 168, 161, 162,
	160, 163, 165, 166, 0, 178, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 55, 56, 164, 0, 0,
	0, 57, 0, 58, 53, 54, 62, 63, 64, 65,
	66, 67, 68, 70, 0, 51, 84, 52, 0, 0,
	0, 0, 24, 76, 77, 0, 0, 0, 69, 46,
	92, 93, 35, 0, 104, 0, 26, 0, 0, 0,
	109, 25, 17, 16, 0, 18, 0, 29, 0, 30,
	0, 0, 19, 0, 0, 0, 20, 21, 34, 36,
	0, 22, 32, 0, 0, 33, 0, 0, 23, 0,
	28, 85, 86, 316, 38, 39, 40, 0, 0, 0,
	50, 108, 0, 101, 97, 98, 99, 94, 95, 0,
	0, 0, 0, 0, 0, 102, 0, 0, 0, 0,
	133, 100, 96, 111, 0, 103, 72, 13, 905, 31,
	59, 0, 0, 105, 87, 88, 89, 90, 91, 0,
	0, 0, 0, 73, 74, 0, 75, 158, 171, 172,
	169, 170, 177, 173, 174, 175, 176, 167, 168, 161,
	162, 160, 163, 165, 166, 0, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 164, 55,
	56, 0, 0, 0, 0, 57, 0, 58, 53, 54,
	62, 63, 64, 65, 66, 67, 68, 70, 0, 51,
	84, 52, 0, 0, 0, 0, 24, 76, 77, 0,
	0, 0, 69, 46, 92, 93, 35, 0, 104, 0,
	26, 0, 0, 0, 109, 25, 17, 16, 0, 18,
	904, 29, 0, 30, 0, 0, 19, 0, 0, 0,
	20, 21, 34, 36, 0, 22, 32, 0, 0, 33,
	0, 0, 23, 0, 28, 85, 86, 316, 38, 39,
	40, 0, 0, 0, 50, 108, 0, 101, 97, 98,
	99, 94, 95, 0, 0, 0, 0, 0, 0, 102,
	0, 0, 0, 0, 133, 100, 96, 111, 0, 103,
	72, 13, 0, 31, 59, 0, 0, 105, 87, 88,
	89, 90, 91, 0, 0, 0, 0, 73, 74, 0,
	75, 171, 172, 169, 170, 177, 173, 174, 175, 176,
	167, 168, 161, 162, 160, 163, 165, 166, 0, 178,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 164, 0, 55, 56, 0, 0, 0, 0, 57,
	0, 58, 53, 54, 62, 63, 64, 65, 66, 67,
	68, 70, 0, 51, 84, 52, 0, 0, 0, 0,
	24, 76, 77, 0, 0, 0, 69, 46, 92, 93,
	35, 0, 104, 0, 26, 0, 0, 0, 109, 25,
	17, 16, 0, 18, 0, 29, 0, 30, 0, 0,
	19, 0, 0, 0, 20, 21, 34, 36, 0, 22,
	32, 0, 0, 33, 0, 0, 23, 0, 28, 85,
	86, 316, 38, 39, 40, 0, 0, 0, 50, 108,
	0, 101, 97, 98, 99, 94, 95, 0, 0, 0,
	0, 0, 0, 102, 0, 0, 0, 0, 133, 100,
	96, 111, 0, 103, 72, 13, 896, 31, 59, 0,
	0, 105, 87, 88, 89, 90, 91, 0, 0, 0,
	0, 73, 74, 0, 75, 173, 174, 175, 176, 167,
	168, 161, 162, 160, 163, 165, 166, 0, 178, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	164, 0, 0, 0, 0, 0, 0, 55, 56, 0,
	0, 0, 0, 57, 0, 58, 53, 54, 62, 63,
	64, 65, 66, 67, 68, 70, 0, 51, 84, 52,
	0, 0, 0, 0, 24, 76, 77, 0, 0, 0,
	69, 46, 92, 93, 35, 0, 104, 0, 26, 0,
	0, 0, 109, 25, 17, 16, 0, 18, 0, 29,
	0, 30, 0, 0, 19, 0, 0, 0, 20, 21,
	34, 36, 0, 22, 32, 0, 0, 33, 0, 0,
	23, 0, 28, 85, 86, 316, 38, 39, 40, 0,
	0, 0, 50, 108, 0, 101, 97, 98, 99, 94,
	95, 0, 0, 0, 0, 0, 0, 102, 0, 0,
	0, 0, 133, 100, 96, 111, 0, 103, 72, 13,
	895, 31, 59, 0, 0, 105, 87, 88, 89, 90,
	91, 0, 0, 0, 0, 73, 74, 0, 75, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 200, 201, 202, 203, 205, 206, 207,
	208, 209, 210, 211, 212, 204, 0, 0, 0, 0,
	0, 55, 56, 0, 0, 0, 0, 57, 0, 58,
	53, 54, 62, 63, 64, 65, 66, 67, 68, 70,
	0, 51, 84, 52, 213, 214, 0, 0, 24, 76,
	77, 0, 0, 0, 69, 46, 92, 93, 35, 0,
	104, 0, 26, 0, 0, 0, 109, 25, 17, 16,
	0, 18, 0, 29, 0, 30, 0, 0, 19, 0,
	0, 0, 20, 21, 34, 36, 0, 22, 32, 0,
	0, 33, 0, 0, 23, 0, 28, 85, 86, 316,
	38, 39, 40, 0, 0, 0, 50, 108, 0, 101,
	97, 98, 99, 94, 95, 0, 0, 0, 0, 0,
	0, 102, 0, 0, 0, 0, 133, 100, 96, 111,
	0, 103, 72, 13, 872, 31, 59, 0, 0, 105,
	87, 88, 89, 90, 91, 0, 0, 0, 0, 73,
	74, 0, 75, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 55, 56, 0, 0, 0,
	0, 57, 0, 58, 53, 54, 62, 63, 64, 65,
	66, 67, 68, 70, 0, 51, 84, 52, 0, 0,
	0, 0, 24, 76, 77, 0, 0, 0, 69, 46,
	92, 93, 35, 0, 104, 0, 26, 0, 0, 0,
	109, 25, 17, 16, 0, 18, 0, 29, 868, 30,
	0, 0, 19, 0, 0, 0, 20, 21, 34, 36,
	0, 22, 32, 0, 0, 33, 0, 0, 23, 0,
	28, 85, 86, 316, 38, 39, 40, 0, 0, 0,
	50, 108, 0, 101, 97, 98, 99, 94, 95, 0,
	0, 0, 0, 0, 0, 102, 0, 0, 0, 0,
	133, 100, 96, 111, 0, 103, 72, 13, 0, 31,
	59, 0, 0, 105, 87, 88, 89, 90, 91, 0,
	0, 0, 0, 73, 74, 0, 75, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 55,
	56, 0, 0, 0, 0, 57, 0, 58, 53, 54,
	62, 63, 64, 65, 66, 67, 68, 70, 0, 51,
	84, 52, 0, 0, 0, 0, 24, 76, 77, 0,
	0, 0, 69, 46, 92, 93, 35, 0, 104, 0,
	26, 0, 0, 0, 109, 25, 17, 16, 0, 18,
	0, 29, 0, 30, 788, 0, 19, 0, 0, 0,
	20, 21, 34, 36, 0, 22, 32, 0, 0, 33,
	0, 0, 23, 0, 28, 85, 86, 316, 38, 39,
	40, 0, 0, 0, 50, 108, 0, 101, 97, 98,
	99, 94, 95, 0, 0, 0, 0, 0, 0, 102,
	0, 0, 0, 0, 133, 100, 96, 111, 0, 103,
	72, 13, 0, 31, 59, 0, 0, 105, 87, 88,
	89, 90, 91, 0, 0, 0, 0, 73, 74, 0,
	75, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 55, 56, 0, 0, 0, 0, 57,
	0, 58, 53, 54, 62, 63, 64, 65, 66, 67,
	68, 70, 0, 51, 84, 52, 0, 0, 0, 0,
	24, 76, 77, 0, 0, 0, 69, 46, 92, 93,
	35, 0, 104, 0, 26, 0, 0, 0, 109, 25,
	17, 16, 771, 18, 0, 29, 0, 30, 0, 0,
	19, 0, 0, 0, 20, 21, 34, 36, 0, 22,
	32, 0, 0, 33, 0, 0, 23, 0, 28, 85,
	86, 316, 38, 39, 40, 0, 0, 0, 50, 108,
	0, 101, 97, 98, 99, 94, 95, 0, 0, 0,
	0, 0, 0, 102, 0, 0, 0, 0, 133, 100,
	96, 111, 0, 103, 72, 13, 0, 31, 59, 0,
	0, 105, 87, 88, 89, 90, 91, 0, 0, 0,
	0, 73, 74, 0, 75, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 55, 56, 0,
	0, 0, 0, 57, 0, 58, 53, 54, 62, 63,
	64, 65, 66, 67, 68, 70, 0, 51, 84, 52,
	0, 0, 0, 694, 24, 76, 77, 0, 0, 0,
	69, 46, 92, 93, 35, 0, 104, 0, 26, 0,
	0, 0, 109, 25, 17, 16, 0, 18, 0, 29,
	0, 30, 0, 0, 19, 0, 0, 0, 20, 21,
	34, 36, 0, 22, 32, 0, 0, 33, 0, 0,
	23, 0, 28, 85, 86, 316, 38, 39, 40, 0,
	0, 0, 50, 108, 0, 101, 97, 98, 99, 94,
	95, 0, 0, 0, 0, 0, 0, 102, 0, 0,
	0, 0, 133, 100, 96, 111, 0, 103, 72, 13,
	0, 31, 59, 0, 0, 105, 87, 88, 89, 90,
	91, 0, 0, 0, 0, 73, 74, 0, 75, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 55, 56, 0, 0, 0, 0, 57, 0, 58,
	53, 54, 62, 63, 64, 65, 66, 67, 68, 70,
	0, 51, 84, 52, 0, 0, 0, 0, 24, 76,
	77, 0, 0, 0, 69, 46, 92, 93, 35, 0,
	104, 0, 26, 0, 0, 0, 109, 25, 17, 16,
	0, 18, 0, 29, 0, 30, 0, 0, 19, 0,
	0, 0, 20, 21, 34, 36, 0, 22, 32, 0,
	0, 33, 0, 0, 23, 0, 28, 85, 86, 316,
	38, 39, 40, 0, 0, 0, 50, 108, 0, 101,
	97, 98, 99, 94, 95, 0, 0, 0, 0, 0,
	0, 102, 0, 0, 0, 0, 133, 100, 96, 111,
	0, 103, 72, 13, 567, 31, 59, 0, 0, 105,
	87, 88, 89, 90, 91, 0, 0, 0, 0, 73,
	74, 0, 75, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 55, 56, 0, 0, 0,
	0, 57, 0, 58, 53, 54, 62, 63, 64, 65,
	66, 67, 68, 70, 0, 51, 84, 52, 0, 0,
	0, 0, 24, 76, 77, 0, 0, 0, 69, 46,
	92, 93, 35, 0, 104, 0, 26, 0, 0, 0,
	109, 25, 17, 16, 0, 18, 0, 29, 0, 30,
	0, 0, 19, 0, 0, 0, 20, 21, 34, 36,
	0, 22, 32, 0, 0, 33, 0, 0, 23, 0,
	28, 85, 86, 316, 38, 39, 40, 0, 0, 0,
	50, 108, 0, 101, 97, 98, 99, 94, 95, 0,
	0, 0, 0, 0, 0, 102, 0, 0, 0, 0,
	133, 100, 96, 111, 0, 103, 72, 13, 310, 31,
	59, 0, 0, 105, 87, 88, 89, 90, 91, 0,
	0, 0, 0, 73, 74, 0, 75, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 55,
	56, 0, 0, 0, 0, 57, 0, 58, 53, 54,
	62, 63, 64, 65, 66, 67, 68, 70, 0, 51,
	84, 52, 0, 0, 0, 0, 24, 76, 77, 0,
	0, 0, 69, 46, 92, 93, 35, 0, 104, 0,
	26, 0, 0, 0, 109, 25, 17, 16, 0, 18,
	0, 29, 0, 30, 0, 0, 19, 0, 0, 0,
	20, 21, 34, 36, 0, 22, 32, 0, 0, 33,
	0, 0, 23, 0, 28, 85, 86, 316, 38, 39,
	40, 0, 0, 0, 50, 108, 0, 101, 97, 98,
	99, 94, 95, 0, 0, 0, 0, 0, 0, 102,
	0, 0, 0, 0, 133, 100, 96, 111, 0, 103,
	72, 13, 0, 31, 59, 156, 155, 105, 441, 442,
	443, 444, 445, 0, 446, 447, 448, 484, 485, 0,
	0, 0, 0, 0, 179, 0, 180, 152, 153, 157,
	159, 158, 171, 172, 169, 170, 177, 173, 174, 175,
	176, 167, 168, 161, 162, 160, 163, 165, 166, 0,
	178, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	449, 0, 164, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 450, 451, 0, 454, 455, 456,
	435, 436, 437, 438, 439, 440, 452, 453, 0, 0,
	432, 0, 104, 0, 0, 0, 0, 0, 0, 457,
	458, 459, 460, 461, 462, 463, 464, 465, 466, 467,
	487, 488, 489, 490, 491, 479, 480, 481, 482, 483,
	468, 469, 470, 471, 472, 473, 474, 475, 476, 477,
	478, 0, 499, 497, 498, 494, 495, 0, 486, 492,
	493, 500, 501, 503, 502, 504, 505, 0, 0, 0,
	0, 0, 87, 88, 89, 90, 91, 0, 496, 507,
	506, 73, 74, 0, 75, 431, 0, 0, 0, 0,
	0, 105, 0, 0, 0, 0, 0, 0, 0, 783,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 55, 56, 0,
	0, 0, 0, 57, 0, 58, 53, 54, 62, 63,
	64, 65, 66, 67, 68, 70, 0, 51, 84, 52,
	0, 0, 0, 0, 24, 0, 0, 0, 0, 0,
	69, 46, 92, 93, 35, 0, 104, 0, 26, 0,
	0, 0, 109, 25, 17, 16, 0, 18, 0, 29,
	0, 30, 0, 0, 19, 0, 0, 0, 20, 21,
	34, 132, 0, 22, 32, 0, 0, 33, 0, 0,
	23, 0, 28, 85, 86, 0, 0, 0, 0, 0,
	0, 0, 50, 108, 0, 101, 97, 98, 99, 94,
	95, 0, 0, 0, 0, 0, 0, 102, 0, 0,
	0, 0, 133, 100, 96, 111, 0, 103, 72, 13,
	0, 31, 59, 0, 0, 105, 87, 88, 89, 90,
	91, 0, 0, 0, 0, 73, 74, 0, 75, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 863, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 55, 56, 0, 0, 0, 0, 57, 0, 58,
	53, 54, 62, 63, 64, 65, 66, 67, 68, 70,
	0, 51, 84, 52, 0, 0, 0, 0, 24, 0,
	0, 0, 0, 0, 69, 46, 92, 93, 35, 0,
	104, 0, 26, 0, 0, 0, 109, 25, 17, 16,
	0, 18, 0, 29, 0, 30, 0, 0, 19, 0,
	0, 0, 20, 21, 34, 132, 0, 22, 32, 0,
	0, 33, 0, 0, 23, 0, 28, 85, 86, 0,
	0, 0, 0, 0, 0, 0, 50, 108, 0, 101,
	97, 98, 99, 94, 95, 0, 0, 0, 0, 0,
	0, 102, 0, 0, 0, 0, 133, 100, 96, 111,
	0, 103, 72, 13, 0, 31, 59, 0, 0, 105,
	87, 88, 89, 90, 91, 0, 0, 0, 0, 73,
	74, 0, 75, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 696, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 55, 56, 0, 0, 0,
	0, 57, 0, 58, 53, 54, 62, 63, 64, 65,
	66, 67, 68, 70, 0, 51, 84, 52, 0, 0,
	0, 0, 24, 0, 0, 0, 0, 0, 69, 46,
	92, 93, 35, 0, 104, 0, 26, 0, 0, 0,
	109, 25, 17, 16, 0, 18, 0, 29, 0, 30,
	0, 0, 19, 0, 0, 0, 20, 21, 34, 132,
	0, 22, 32, 0, 0, 33, 0, 0, 23, 0,
	28, 85, 86, 0, 0, 0, 0, 0, 0, 0,
	50, 108, 0, 101, 97, 98, 99, 94, 95, 0,
	0, 0, 0, 0, 0, 102, 0, 0, 0, 0,
	133, 100, 96, 111, 0, 103, 72, 13, 0, 31,
	59, 0, 0, 105, 87, 88, 89, 90, 91, 0,
	0, 0, 0, 73, 74, 0, 75, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 665, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 55,
	56, 0, 0, 0, 0, 57, 0, 58, 53, 54,
	62, 63, 64, 65, 66, 67, 68, 70, 0, 51,
	84, 52, 0, 0, 0, 0, 24, 0, 0, 0,
	0, 0, 69, 46, 92, 93, 35, 0, 104, 0,
	26, 0, 0, 0, 109, 25, 17, 16, 0, 18,
	0, 29, 0, 30, 0, 0, 19, 0, 0, 0,
	20, 21, 34, 132, 0, 22, 32, 0, 0, 33,
	0, 0, 23, 0, 28, 85, 86, 0, 0, 0,
	0, 0, 0, 0, 50, 108, 0, 101, 97, 98,
	99, 94, 95, 0, 0, 0, 0, 0, 0, 102,
	0, 0, 0, 0, 133, 100, 96, 111, 0, 103,
	72, 13, 0, 31, 59, 0, 0, 105, 87, 88,
	89, 90, 91, 0, 0, 0, 0, 73, 74, 0,
	75, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 647, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 55, 56, 0, 0, 0, 0, 57,
	0, 58, 53, 54, 62, 63, 64, 65, 66, 67,
	68, 70, 0, 51, 84, 52, 0, 0, 0, 0,
	24, 0, 0, 0, 0, 0, 69, 46, 92, 93,
	35, 0, 104, 0, 26, 0, 0, 0, 109, 25,
	17, 16, 0, 18, 0, 29, 0, 30, 0, 0,
	19, 0, 0, 0, 20, 21, 34, 132, 0, 22,
	32, 0, 0, 33, 0, 0, 23, 0, 28, 85,
	86, 0, 0, 0, 0, 0, 0, 0, 50, 108,
	0, 101, 97, 98, 99, 94, 95, 0, 0, 0,
	0, 0, 0, 102, 0, 0, 0, 0, 133, 100,
	96, 111, 0, 103, 72, 13, 155, 31, 59, 0,
	0, 105, 441, 442, 443, 444, 445, 0, 446, 447,
	448, 484, 485, 0, 179, 0, 180, 152, 153, 157,
	159, 158, 171, 172, 169, 170, 177, 173, 174, 175,
	176, 167, 168, 161, 162, 160, 163, 165, 166, 0,
	178, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 164, 0, 449, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 450, 451,
	0, 454, 455, 456, 435, 436, 437, 438, 439, 440,
	452, 453, 0, 0, 858, 0, 0, 0, 0, 0,
	0, 0, 0, 457, 458, 459, 460, 461, 462, 463,
	464, 465, 466, 467, 487, 488, 489, 490, 491, 479,
	480, 481, 482, 483, 468, 469, 470, 471, 472, 473,
	474, 475, 476, 477, 478, 0, 499, 497, 498, 494,
	495, 0, 486, 492, 493, 500, 501, 503, 502, 504,
	505, 441, 442, 443, 444, 445, 426, 446, 447, 448,
	484, 485, 857, 507, 506, 111, 0, 0, 0, 0,
	882, 179, 0, 180, 152, 153, 157, 159, 158, 171,
	172, 169, 170, 177, 173, 174, 175, 176, 167, 168,
	161, 162, 160, 163, 165, 166, 0, 178, 0, 0,
	0, 0, 0, 449, 0, 0, 0, 0, 0, 164,
	0, 0, 0, 0, 0, 0, 0, 450, 451, 0,
	454, 455, 456, 435, 436, 437, 438, 439, 440, 452,
	453, 0, 0, 858, 0, 0, 0, 0, 0, 0,
	0, 0, 457, 458, 459, 460, 461, 462, 463, 464,
	465, 466, 467, 487, 488, 489, 490, 491, 479, 480,
	481, 482, 483, 468, 469, 470, 471, 472, 473, 474,
	475, 476, 477, 478, 0, 499, 497, 498, 494, 495,
	0, 486, 492, 493, 500, 501, 503, 502, 504, 505,
	154, 156, 155, 0, 0, 0, 87, 88, 89, 90,
	91, 857, 507, 506, 111, 73, 74, 0, 75, 848,
	179, 0, 180, 152, 153, 157, 159, 158, 171, 172,
	169, 170, 177, 173, 174, 175, 176, 167, 168, 161,
	162, 160, 163, 165, 166, 0, 178, 0, 0, 0,
	0, 55, 56, 0, 0, 0, 0, 57, 164, 58,
	53, 54, 62, 63, 64, 65, 66, 67, 68, 70,
	0, 51, 84, 52, 0, 0, 0, 0, 24, 0,
	0, 0, 0, 0, 69, 46, 92, 93, 35, 0,
	104, 0, 26, 0, 0, 0, 109, 25, 17, 16,
	0, 18, 0, 29, 0, 30, 0, 0, 19, 0,
	0, 0, 20, 21, 34, 132, 0, 22, 32, 0,
	0, 33, 0, 0, 23, 0, 28, 85, 86, 0,
	0, 0, 0, 0, 0, 0, 50, 108, 0, 101,
	97, 98, 99, 94, 95, 0, 0, 0, 0, 0,
	0, 102, 0, 0, 0, 693, 133, 100, 96, 111,
	0, 103, 72, 13, 0, 31, 59, 0, 0, 105,
	441, 442, 443, 444, 445, 0, 446, 447, 448, 484,
	485, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 449, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 450, 451, 0, 454,
	455, 456, 435, 436, 437, 438, 439, 440, 452, 453,
	0, 0, 432, 0, 0, 0, 0, 0, 0, 0,
	0, 457, 458, 459, 460, 461, 462, 463, 464, 465,
	466, 467, 487, 488, 489, 490, 491, 479, 480, 481,
	482, 483, 468, 469, 470, 471, 472, 473, 474, 475,
	476, 477, 478, 0, 499, 497, 498, 494, 495, 0,
	486, 492, 493, 500, 501, 503, 502, 504, 505, 441,
	442, 443, 444, 445, 0, 446, 447, 448, 484, 485,
	496, 507, 506, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 449, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 450, 451, 0, 454, 455,
	456, 687, 688, 689, 686, 685, 684, 452, 453, 0,
	0, 900, 0, 0, 0, 0, 0, 0, 0, 0,
	457, 458, 459, 460, 461, 462, 463, 464, 465, 466,
	467, 487, 488, 489, 490, 491, 479, 480, 481, 482,
	483, 468, 469, 470, 471, 472, 473, 474, 475, 476,
	477, 478, 0, 499, 497, 498, 494, 495, 0, 486,
	492, 493, 500, 501, 503, 502, 504, 505, 0, 0,
	0, 233, 0, 87, 88, 89, 90, 91, 232, 496,
	507, 506, 73, 74, 0, 75, 0, 0, 116, 0,
	104, 0, 0, 0, 0, 0, 109, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 55, 56,
	0, 0, 0, 0, 57, 0, 58, 53, 54, 62,
	63, 64, 65, 66, 67, 68, 70, 108, 51, 84,
	52, 0, 0, 0, 0, 138, 0, 0, 0, 0,
	0, 69, 0, 92, 93, 116, 133, 104, 0, 111,
	0, 0, 0, 109, 0, 0, 230, 0, 0, 105,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 132, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 85, 86, 0, 0, 0, 0,
	0, 0, 0, 50, 108, 0, 101, 97, 98, 99,
	94, 95, 0, 0, 0, 0, 0, 0, 102, 0,
	0, 0, 0, 133, 100, 96, 111, 532, 103, 72,
	0, 0, 0, 59, 528, 0, 105, 87, 88, 89,
	90, 91, 0, 0, 0, 0, 73, 74, 0, 75,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	225, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 55, 56, 0, 0, 0, 0, 57, 0,
	58, 53, 54, 62, 63, 64, 65, 66, 67, 68,
	70, 0, 51, 84, 52, 0, 0, 0, 0, 138,
	0, 0, 0, 0, 0, 69, 0, 92, 93, 116,
	0, 104, 0, 0, 0, 0, 0, 109, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 132, 0, 0, 0,
	0, 0, 0, 87, 88, 89, 90, 91, 85, 86,
	0, 0, 73, 74, 0, 75, 0, 226, 108, 0,
	101, 97, 98, 99, 94, 95, 0, 0, 0, 0,
	0, 0, 102, 0, 0, 0, 0, 133, 100, 96,
	111, 0, 103, 72, 0, 0, 0, 59, 55, 56,
	105, 0, 0, 0, 57, 0, 58, 53, 54, 62,
	63, 64, 65, 66, 67, 68, 70, 0, 51, 84,
	52, 0, 0, 0, 0, 138, 0, 0, 0, 0,
	0, 69, 0, 92, 93, 116, 0, 104, 0, 0,
	0, 0, 0, 109, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 132, 0, 0, 0, 0, 0, 0, 87,
	88, 89, 90, 91, 85, 86, 0, 0, 73, 74,
	0, 75, 0, 50, 108, 0, 101, 97, 98, 99,
	94, 95, 0, 0, 0, 0, 0, 0, 102, 0,
	0, 0, 600, 133, 100, 96, 111, 532, 103, 72,
	0, 0, 0, 59, 55, 56, 105, 0, 0, 0,
	57, 0, 58, 53, 54, 62, 63, 64, 65, 66,
	67, 68, 70, 0, 51, 84, 52, 0, 0, 0,
	0, 138, 0, 0, 0, 0, 0, 69, 0, 92,
	93, 116, 0, 104, 0, 0, 0, 0, 0, 109,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 132, 0,
	0, 0, 0, 0, 0, 87, 88, 89, 90, 91,
	85, 86, 0, 0, 73, 74, 0, 75, 0, 601,
	108, 0, 101, 97, 98, 99, 94, 95, 0, 0,
	0, 0, 0, 0, 102, 0, 0, 0, 0, 133,
	100, 96, 111, 0, 103, 72, 0, 0, 0, 59,
	55, 56, 105, 0, 0, 0, 57, 0, 58, 53,
	54, 62, 63, 64, 65, 66, 67, 68, 70, 0,
	51, 84, 52, 0, 0, 0, 0, 138, 0, 0,
	0, 0, 0, 69, 0, 92, 93, 116, 424, 104,
	0, 0, 0, 0, 0, 109, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 132, 0, 0, 0, 0, 0,
	0, 87, 88, 89, 90, 91, 85, 86, 0, 0,
	73, 74, 0, 75, 0, 50, 108, 0, 101, 97,
	98, 99, 94, 95, 0, 0, 0, 0, 0, 0,
	102, 0, 0, 0, 387, 133, 100, 96, 111, 0,
	103, 72, 0, 0, 0, 59, 55, 56, 105, 0,
	0, 0, 57, 0, 58, 53, 54, 62, 63, 64,
	65, 66, 67, 68, 70, 0, 51, 84, 52, 0,
	0, 0, 0, 138, 0, 0, 0, 0, 0, 69,
	0, 92, 93, 116, 0, 104, 0, 0, 0, 0,
	0, 109, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	132, 0, 0, 0, 0, 0, 0, 87, 88, 89,
	90, 91, 85, 86, 0, 0, 73, 74, 0, 75,
	0, 50, 108, 0, 101, 97, 98, 99, 94, 95,
	0, 0, 0, 0, 363, 0, 102, 0, 0, 0,
	0, 133, 100, 96, 111, 0, 103, 72, 0, 0,
	0, 59, 55, 56, 105, 0, 0, 0, 57, 0,
	58, 53, 54, 62, 63, 64, 65, 66, 67, 68,
	70, 0, 51, 84, 52, 0, 0, 0, 0, 138,
	0, 0, 0, 0, 0, 69, 0, 92, 93, 116,
	0, 104, 0, 0, 0, 0, 0, 109, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 132, 0, 0, 0,
	0, 0, 0, 87, 88, 89, 90, 91, 85, 86,
	0, 0, 73, 74, 0, 75, 0, 50, 108, 0,
	101, 97, 98, 99, 94, 95, 0, 0, 0, 0,
	0, 0, 102, 0, 0, 0, 0, 133, 100, 96,
	111, 0, 103, 72, 0, 0, 0, 59, 55, 56,
	105, 0, 0, 0, 57, 0, 58, 53, 54, 62,
	63, 64, 65, 66, 67, 68, 70, 0, 51, 84,
	52, 0, 0, 0, 0, 138, 0, 0, 0, 0,
	0, 69, 0, 92, 93, 116, 0, 104, 0, 0,
	0, 0, 0, 109, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 132, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 85, 86, 0, 0, 0, 0,
	0, 0, 0, 50, 108, 0, 101, 97, 98, 99,
	94, 95, 0, 0, 0, 154, 156, 155, 102, 0,
	0, 0, 0, 133, 100, 96, 111, 0, 103, 72,
	0, 0, 0, 59, 0, 179, 105, 180, 152, 153,
	157, 159, 158, 171, 172, 169, 170, 177, 173, 174,
	175, 176, 167, 168, 161, 162, 160, 163, 165, 166,
	0, 178, 154, 156, 155, 0, 0, 0, 0, 0,
	0, 0, 0, 164, 0, 0, 0, 0, 0, 0,
	0, 0, 179, 0, 180, 152, 153, 157, 159, 158,
	171, 172, 169, 170, 177, 173, 174, 175, 176, 167,
	168, 161, 162, 160, 163, 165, 166, 0, 178, 154,
	156, 155, 0, 0, 0, 0, 0, 0, 0, 0,
	164, 0, 0, 0, 0, 0, 0, 0, 0, 179,
	0, 180, 152, 153, 157, 159, 158, 171, 172, 169,
	170, 177, 173, 174, 175, 176, 167, 168, 161, 162,
	160, 163, 165, 166, 0, 178, 154, 156, 155, 0,
	0, 0, 0, 0, 0, 0, 0, 164, 0, 0,
	0, 763, 0, 0, 0, 0, 179, 0, 180, 152,
	153, 157, 159, 158, 171, 172, 169, 170, 177, 173,
	174, 175, 176, 167, 168, 161, 162, 160, 163, 165,
	166, 0, 178, 154, 156, 155, 0, 0, 0, 0,
	0, 0, 0, 0, 164, 0, 0, 692, 0, 0,
	0, 0, 0, 179, 0, 180, 152, 153, 157, 159,
	158, 171, 172, 169, 170, 177, 173, 174, 175, 176,
	167, 168, 161, 162, 160, 163, 165, 166, 0, 178,
	154, 156, 155, 0, 0, 0, 0, 0, 0, 0,
	0, 164, 0, 0, 630, 0, 0, 0, 0, 0,
	179, 0, 180, 152, 153, 157, 159, 158, 171, 172,
	169, 170, 177, 173, 174, 175, 176, 167, 168, 161,
	162, 160, 163, 165, 166, 0, 178, 154, 156, 155,
	0, 0, 0, 0, 0, 0, 0, 0, 164, 0,
	0, 629, 0, 0, 0, 0, 0, 179, 0, 180,
	152, 153, 157, 159, 158, 171, 172, 169, 170, 177,
	173, 174, 175, 176, 167, 168, 161, 162, 160, 163,
	165, 166, 0, 178, 154, 156, 155, 0, 0, 0,
	0, 0, 0, 0, 0, 164, 0, 0, 603, 0,
	0, 0, 0, 0, 179, 0, 180, 152, 153, 157,
	159, 158, 171, 172, 169, 170, 177, 173, 174, 175,
	176, 167, 168, 161, 162, 160, 163, 165, 166, 0,
	178, 154, 156, 155, 0, 0, 0, 0, 0, 0,
	0, 0, 164, 0, 0, 591, 0, 0, 0, 0,
	0, 179, 0, 180, 152, 153, 157, 159, 158, 171,
	172, 169, 170, 177, 173, 174, 175, 176, 167, 168,
	161, 162, 160, 163, 165, 166, 0, 178, 154, 156,
	155, 0, 0, 0, 0, 0, 0, 0, 0, 164,
	0, 0, 555, 0, 0, 0, 0, 0, 179, 820,
	180, 152, 153, 157, 159, 158, 171, 172, 169, 170,
	177, 173, 174, 175, 176, 167, 168, 161, 162, 160,
	163, 165, 166, 0, 178, 154, 156, 155, 0, 0,
	0, 0, 0, 0, 0, 0, 164, 0, 0, 551,
	0, 0, 0, 0, 0, 179, 0, 180, 152, 153,
	157, 159, 158, 171, 172, 169, 170, 177, 173, 174,
	175, 176, 167, 168, 161, 162, 160, 163, 165, 166,
	0, 178, 154, 156, 155, 0, 0, 0, 0, 0,
	0, 0, 0, 164, 0, 0, 415, 0, 0, 0,
	0, 0, 179, 0, 180, 152, 153, 157, 159, 158,
	171, 172, 169, 170, 177, 173, 174, 175, 176, 167,
	168, 161, 162, 160, 163, 165, 166, 0, 178, 154,
	156, 155, 0, 0, 0, 0, 0, 0, 0, 0,
	164, 821, 0, 0, 0, 0, 0, 0, 0, 179,
	0, 180, 152, 153, 157, 159, 158, 171, 172, 169,
	170, 177, 173, 174, 175, 176, 167, 168, 161, 162,
	160, 163, 165, 166, 0, 178, 154, 156, 155, 0,
	0, 0, 0, 0, 0, 0, 0, 164, 371, 0,
	0, 0, 0, 0, 0, 0, 179, 0, 180, 152,
	153, 157, 159, 158, 171, 172, 169, 170, 177, 173,
	174, 175, 176, 167, 168, 161, 162, 160, 163, 165,
	166, 0, 178, 0, 154, 156, 155, 0, 0, 0,
	0, 0, 0, 0, 164, 151, 0, 0, 0, 0,
	0, 0, 0, 0, 179, 0, 180, 152, 153, 157,
	159, 158, 171, 172, 169, 170, 177, 173, 174, 175,
	176, 167, 168, 161, 162, 160, 163, 165, 166, 0,
	178, 154, 156, 155, 0, 0, 0, 0, 0, 0,
	0, 708, 164, 0, 0, 0, 0, 0, 0, 0,
	0, 179, 0, 180, 152, 153, 157, 159, 158, 171,
	172, 169, 170, 177, 173, 174, 175, 176, 167, 168,
	161, 162, 160, 163, 165, 166, 0, 178, 0, 154,
	156, 155, 0, 0, 0, 0, 0, 0, 706, 164,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 179,
	0, 180, 152, 153, 157, 159, 158, 171, 172, 169,
	170, 177, 173, 174, 175, 176, 167, 168, 161, 162,
	160, 163, 165, 166, 0, 178, 154, 156, 155, 0,
	0, 0, 0, 0, 0, 0, 697, 164, 0, 0,
	0, 0, 0, 0, 0, 0, 179, 0, 180, 152,
	153, 157, 159, 158, 171, 172, 169, 170, 177, 173,
	174, 175, 176, 167, 168, 161, 162, 160, 163, 165,
	166, 0, 178, 0, 154, 156, 155, 0, 0, 0,
	0, 0, 0, 631, 164, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 179, 0, 180, 152, 153, 157,
	159, 158, 171, 172, 169, 170, 177, 173, 174, 175,
	176, 167, 168, 161, 162, 160, 163, 165, 166, 0,
	178, 154, 156, 155, 0, 0, 0, 0, 0, 0,
	0, 612, 164, 0, 0, 0, 0, 0, 0, 0,
	0, 179, 561, 180, 152, 153, 157, 159, 158, 171,
	172, 169, 170, 177, 173, 174, 175, 176, 167, 168,
	161, 162, 160, 163, 165, 166, 0, 178, 0, 565,
	154, 156, 155, 0, 0, 410, 0, 0, 593, 164,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	179, 0, 180, 152, 153, 157, 159, 158, 171, 172,
	169, 170, 177, 173, 174, 175, 176, 167, 168, 161,
	162, 160, 163, 165, 166, 0, 178, 154, 156, 155,
	0, 0, 0, 0, 0, 0, 0, 0, 164, 0,
	0, 0, 0, 0, 0, 0, 0, 179, 0, 180,
	152, 153, 157, 159, 158, 171, 172, 169, 170, 177,
	173, 174, 175, 176, 167, 168, 161, 162, 160, 163,
	165, 166, 0, 178, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 164,
}
var yyPact = [...]int{

	-1000, -1000, 1194, -1000, -1000, -1000, -1000, -1000, -1000, 149,
	251, 421, 516, -1000, -1000, -1000, 146, 4902, 144, 143,
	6209, 6209, 6209, 20, 452, 6209, -1000, 6862, 141, 138,
	137, -1000, 259, 6209, 584, 698, 651, 573, 574, 571,
	568, 607, 723, 2356, -1000, -1000, 134, -1000, -1000, 302,
	132, 5513, 6209, 5282, 5282, 6209, 6209, 6209, 6209, 6209,
	-1000, -1000, 6209, 6209, 6209, 6209, 6209, 6209, 6209, 131,
	6209, -1000, 638, 6209, 6209, 6209, -1000, -1000, -1000, 614,
	-1000, 364, 361, -1000, 172, 128, 105, 6209, 6209, 95,
	6209, 6209, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, 540, 635, -1000, 33, 66, 66, 83, -1000,
	272, 498, -22, 498, -12, -1000, -1000, 206, 365, 290,
	438, 498, -1000, -1000, -1000, -1000, 276, -1000, 737, 3326,
	6209, 445, 651, 271, 6209, 6209, 204, 7337, 417, 203,
	199, 267, -1000, -1000, 261, 651, -1000, 732, 237, -1000,
	7337, -1000, 6209, 6209, 6209, 6209, 6209, 6209, 6209, 6209,
	6209, 6209, 6209, 6209, 6209, 6209, 6209, 6209, 6209, 6209,
	6209, 6209, 6209, 6209, 6209, 6209, 6209, 6209, 31, 6093,
	6209, 5282, 6209, 516, -1000, 6815, 197, -1000, 58, -1000,
	566, -1000, 324, 258, 390, 81, 4902, 78, 195, 682,
	5977, 6209, 6209, 6209, 6209, 6209, 6209, 6209, 6209, 6209,
	6209, 6209, 6209, -1000, -1000, 6209, 6209, 6209, 96, 5513,
	-61, 793, -1000, -1000, 7290, 5282, 77, -1000, -1000, 614,
	6209, -1000, -1000, 5513, -1000, 618, 618, 656, 618, 6721,
	618, 618, 618, 618, 618, 618, 618, -1000, 6209, 618,
	262, 612, 553, -1000, 456, 5861, 5282, 1015, 4751, 1015,
	6209, 3654, 3654, 66, -1000, 301, 198, 66, -1000, -1000,
	6209, 6209, 7337, 7337, 6209, 7337, 7337, 567, -1000, 459,
	368, 612, 6209, -1000, -1000, 5349, -1000, 5513, 561, 272,
	193, 272, -1000, -1000, 1030, -1000, 189, 235, 434, 498,
	-1000, 351, 352, 557, 413, -1000, -1000, 516, 6209, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, 76, 6674, 75, 74,
	185, 791, 7337, 6627, -1000, -1000, -1000, -1000, 20, -1000,
	487, 69, 6209, -1000, 6209, 1341, 1505, 3644, 1015, 4604,
	1669, 1993, 1830, 846, 846, 846, 656, 618, 656, 656,
	1288, 1288, 2152, 2152, 2152, 2152, 776, 776, 776, 776,
	2152, -1000, 7241, 6209, 1179, 784, -1000, -1000, 7194, 103,
	3162, -1000, -1000, 68, 539, 324, 299, 451, -1000, 256,
	451, 6209, -1000, 6209, -1000, -1000, 1015, 6209, 1015, 1015,
	1015, 1015, 1015, 1015, 1015, 1015, 1015, 1015, 1015, 1015,
	6580, -62, 7146, 66, -1000, 6209, -1000, -24, 726, 5513,
	5745, -1000, 5513, 6533, -66, -1000, -27, -1000, -1000, -1000,
	-1000, 556, 507, 7099, 278, 229, 6209, -70, 66, -1000,
	-1000, 6209, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, 66, -1000,
	-1000, -1000, -1000, 20, 6209, 6209, 96, 20, 324, 782,
	-1000, 7337, 6486, 6439, -1000, -1000, -1000, 7051, -1000, 772,
	-1000, 7337, 6209, -31, -1000, -1000, 866, -1000, -1000, -1000,
	348, 406, -1000, 498, 353, 623, -1000, 332, -1000, 7337,
	-32, 4454, 6209, 6209, 6209, 330, -1000, -1000, 539, 7337,
	-1000, 6209, 1179, -33, 5282, 269, 4290, -1000, 539, -35,
	766, -1000, 652, -1000, -1000, 446, -1000, -1000, -1000, 299,
	253, 451, -1000, 559, -1000, 748, -1000, 6392, 4890, 2998,
	1993, 4126, -1000, -1000, -1000, 7004, 713, 6209, -1000, 7337,
	5282, 64, -36, -1000, -1000, -1000, -71, -1000, -1000, 455,
	-1000, -1000, -1000, -1000, 6209, -1000, 1015, -1000, -1000, 6956,
	-1000, -1000, -73, 6909, -1000, -1000, 299, -37, 6209, -1000,
	-1000, -1000, -38, 5629, 7337, -1000, -1000, 498, 329, 753,
	-1000, -1000, 498, 623, 180, -1000, -1000, -1000, 432, 179,
	7337, -1000, 178, 177, -39, 1179, 171, -1000, -40, 740,
	5282, 61, 5513, -1000, -1000, -1000, 411, -41, 404, 539,
	263, -1000, -1000, 248, -1000, 748, -1000, -1000, 470, 437,
	451, 693, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	425, 451, 4902, 679, 164, -1000, -1000, -1000, 6209, 1015,
	-1000, 5513, 713, -1000, -1000, 6345, -1000, -1000, -1000, 246,
	-1000, -1000, -1000, -1000, 752, -1000, 498, 228, 623, -1000,
	753, -1000, 2834, 162, 6209, 127, -1000, 523, -1000, 404,
	-1000, 3798, 269, -1000, 5513, -79, 2670, -1000, 59, 244,
	666, 666, 51, -1000, 483, -1000, -1000, 418, 205, -1000,
	724, 5066, 651, 130, -1000, -1000, -1000, -1000, -1000, -1000,
	3490, 1015, -42, 223, -1000, 218, 498, 752, -1000, -1000,
	211, 161, -1000, -43, -1000, 6209, 121, 118, 158, 515,
	666, -1000, -1000, -1000, -45, -1000, -47, -1000, 157, 451,
	-1000, 241, 539, 238, 639, 721, 369, -1000, -1000, 470,
	6209, 190, -1000, 716, 5066, -1000, -1000, 4757, 3490, 713,
	-1000, 174, -1000, -1000, 208, -1000, -1000, 3962, 6768, -1000,
	-1000, -1000, -1000, -1000, 156, 233, 2506, 3798, -1000, -1000,
	511, -1000, 2342, -1000, -1000, -1000, 97, -1000, -1000, 468,
	6209, -1000, -1000, 7337, -1000, 5066, 6209, 25, -1000, 4618,
	-1000, 154, 153, 402, 431, 288, -1000, 271, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, 3490, -1000, -1000, 151, -1000,
	-51, 451, -1000, 2178, 2014, -1000, 639, -1000, 7337, -1000,
	7337, 539, -1000, -1000, -1000, -1000, 451, 5205, 5066, 1850,
	3490, 1686, -1000, 232, -1000, -1000, -1000, -1000, -52, 748,
	-1000, -1000, 5066, -1000, 32, -1000, -1000, 666, -1000, -1000,
	1522, -15, -1000, -1000, -1000, -1000, 1358, -1000,
}
var yyPgo = [...]int{

	0, 940, 939, 46, 6, 938, 3, 934, 14, 27,
	79, 77, 70, 51, 922, 20, 920, 94, 15, 78,
	919, 0, 62, 918, 913, 11, 912, 911, 36, 61,
	49, 33, 909, 35, 907, 72, 50, 905, 9, 904,
	903, 897, 895, 13, 48, 890, 888, 111, 98, 197,
	887, 884, 883, 8, 881, 81, 43, 878, 55, 47,
	877, 876, 875, 868, 867, 97, 863, 862, 861, 860,
	10, 858, 857, 42, 32, 41, 2, 16, 740, 5,
	765, 17, 664, 854, 28, 852, 851, 850, 90, 849,
	31, 848, 846, 25, 29, 843, 842, 840, 24, 838,
	836, 834, 549, 833, 18, 820, 819, 44, 86, 83,
	818, 103, 1, 7, 817, 19, 816, 815, 38, 814,
	812, 810, 4, 806, 803, 37, 12,
}
var yyR1 = [...]int{

	0, 124, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 5,
	5, 5, 5, 5, 5, 5, 6, 6, 111, 111,
	84, 84, 8, 8, 8, 7, 7, 7, 7, 7,
	7, 7, 7, 7, 7, 7, 7, 7, 7, 80,
	80, 14, 14, 16, 16, 125, 125, 94, 94, 93,
	93, 102, 102, 15, 15, 18, 18, 17, 17, 88,
	88, 112, 112, 20, 20, 20, 20, 20, 20, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 86, 86, 85, 85, 27, 27, 101,
	101, 28, 10, 1, 1, 2, 2, 11, 11, 123,
	123, 78, 78, 12, 13, 30, 30, 117, 117, 118,
	118, 81, 81, 81, 81, 24, 24, 25, 25, 26,
	26, 97, 97, 97, 97, 98, 98, 98, 126, 126,
	23, 23, 69, 69, 67, 67, 71, 71, 68, 68,
	113, 113, 114, 114, 31, 31, 32, 32, 77, 77,
	75, 75, 75, 76, 76, 109, 109, 110, 110, 33,
	33, 92, 92, 35, 96, 96, 36, 36, 115, 115,
	37, 37, 37, 37, 122, 122, 100, 100, 100, 99,
	99, 38, 38, 39, 40, 40, 40, 40, 42, 42,
	41, 116, 116, 120, 120, 119, 119, 121, 121, 79,
	79, 79, 79, 79, 79, 95, 95, 43, 43, 87,
	87, 70, 19, 89, 89, 44, 90, 90, 91, 91,
	46, 45, 45, 34, 34, 34, 34, 34, 34, 34,
	34, 34, 34, 34, 34, 34, 34, 34, 34, 34,
	34, 34, 34, 34, 34, 34, 34, 34, 34, 34,
	34, 34, 34, 34, 34, 34, 34, 34, 34, 34,
	34, 34, 34, 34, 34, 34, 34, 34, 34, 34,
	34, 34, 34, 34, 34, 34, 34, 34, 34, 34,
	34, 34, 34, 34, 34, 34, 34, 34, 34, 34,
	34, 34, 34, 34, 34, 34, 34, 34, 34, 3,
	3, 104, 104, 103, 103, 53, 53, 54, 54, 54,
	54, 47, 47, 48, 48, 51, 51, 83, 83, 83,
	108, 108, 58, 58, 58, 52, 52, 52, 52, 52,
	52, 52, 52, 52, 52, 52, 52, 52, 52, 52,
	52, 59, 59, 59, 21, 21, 22, 22, 57, 60,
	60, 60, 61, 61, 61, 62, 62, 62, 62, 62,
	62, 29, 29, 29, 49, 49, 49, 63, 63, 64,
	64, 64, 64, 64, 64, 55, 55, 55, 56, 56,
	56, 107, 73, 73, 106, 106, 72, 72, 72, 72,
	72, 72, 82, 82, 82, 82, 65, 65, 65, 65,
	65, 65, 65, 66, 66, 66, 66, 50, 50, 50,
	50, 50, 50, 50, 105, 105, 74,
}
var yyR2 = [...]int{

	0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 2, 0,
	1, 3, 1, 3, 2, 1, 1, 1, 1, 1,
	4, 3, 5, 4, 3, 4, 3, 4, 3, 1,
	1, 6, 7, 6, 7, 0, 1, 3, 1, 3,
	1, 3, 1, 1, 2, 1, 3, 1, 2, 3,
	1, 2, 0, 1, 1, 1, 1, 1, 4, 3,
	1, 1, 5, 7, 9, 5, 3, 3, 3, 3,
	3, 3, 1, 2, 6, 7, 9, 5, 1, 6,
	3, 3, 2, 0, 9, 1, 3, 0, 4, 1,
	3, 1, 10, 0, 1, 0, 1, 8, 7, 1,
	2, 1, 1, 5, 6, 0, 2, 0, 2, 0,
	2, 1, 2, 4, 3, 1, 4, 1, 4, 1,
	4, 3, 4, 4, 5, 0, 5, 4, 1, 1,
	1, 4, 5, 6, 1, 3, 6, 7, 3, 6,
	1, 0, 1, 3, 4, 6, 0, 1, 1, 2,
	1, 1, 1, 0, 2, 2, 4, 1, 3, 1,
	2, 3, 1, 1, 3, 1, 1, 3, 2, 0,
	3, 4, 3, 9, 1, 3, 1, 2, 3, 1,
	2, 2, 2, 3, 3, 3, 4, 3, 1, 1,
	3, 1, 3, 1, 1, 0, 1, 1, 2, 1,
	1, 1, 1, 1, 1, 3, 1, 1, 3, 3,
	1, 3, 3, 3, 1, 1, 0, 1, 3, 1,
	7, 3, 2, 6, 5, 3, 4, 2, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	2, 2, 2, 2, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 2, 2, 2, 2, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 1, 5, 4, 3,
	1, 2, 2, 2, 2, 2, 2, 2, 2, 2,
	1, 3, 2, 1, 2, 4, 2, 10, 11, 0,
	1, 0, 4, 3, 1, 1, 2, 2, 4, 4,
	2, 1, 1, 1, 1, 0, 3, 0, 1, 1,
	0, 1, 4, 3, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 3, 2, 3, 3, 1,
	1, 1, 3, 3, 1, 1, 0, 1, 1, 1,
	3, 1, 1, 3, 1, 1, 4, 4, 4, 4,
	1, 1, 1, 3, 1, 4, 2, 3, 3, 1,
	4, 4, 3, 3, 3, 1, 3, 1, 1, 3,
	1, 1, 0, 1, 3, 1, 3, 1, 4, 2,
	6, 4, 2, 2, 1, 2, 1, 4, 3, 3,
	3, 6, 3, 1, 1, 2, 1, 5, 4, 2,
	2, 4, 2, 2, 1, 3, 1,
}
var yyChk = [...]int{

	-1000, -124, -111, -7, -9, -10, -11, -12, -13, 127,
	154, 120, 114, 161, -67, -68, 97, 96, 99, 106,
	110, 111, 115, 122, 76, 95, 90, -21, 124, 101,
	103, 163, 116, 119, 112, 86, 113, -123, 128, 129,
	130, -69, -71, -29, -34, -78, 83, -62, -63, -60,
	134, 69, 71, 58, 59, 49, 50, 55, 57, 164,
	-45, -50, 60, 61, 62, 63, 64, 65, 66, 82,
	67, -52, 160, 13, 14, 16, 77, 78, -49, -59,
	-54, -47, -57, -58, 70, 125, 126, 4, 5, 6,
	7, 8, 84, 85, 141, 142, 156, 138, 139, 140,
	155, 137, 149, 159, 88, 167, -8, -61, 135, 94,
	-84, 157, 164, 157, -84, 161, 86, -16, -80, -102,
	-84, 157, 113, 114, -17, -18, -88, -19, 86, -112,
	164, -9, 113, 154, 164, 164, -22, -21, 76, -22,
	-22, -92, -35, -49, -96, 113, -36, 88, -89, -44,
	-21, 163, 33, 34, 10, 12, 11, 35, 37, 36,
	51, 49, 50, 52, 68, 53, 54, 47, 48, 40,
	41, 38, 39, 43, 44, 45, 46, 42, 56, 30,
	32, 164, 164, 164, 161, -21, 86, 31, -3, 37,
	128, -78, 86, 86, 86, 73, 74, 73, 75, 74,
	17, 18, 19, 20, 29, 21, 22, 23, 24, 25,
	26, 27, 28, 58, 59, 164, 69, 161, 133, 164,
	-107, -106, -73, -72, -21, 37, 134, -21, -29, -59,
	164, -58, 76, 69, -29, -21, -21, -21, -21, -21,
	-21, -21, -21, -21, -21, -21, -21, -51, 164, -21,
	-83, 93, -82, -65, 88, 151, 152, -21, -21, -21,
	69, 153, 153, -48, -46, -47, -64, 128, -8, -49,
	164, 164, -21, -21, 164, -21, -21, 93, 150, -82,
	-82, 93, 161, -49, -109, 164, -109, 164, 157, -84,
	165, -84, 163, 161, -111, 163, -14, -102, -84, 157,
	163, 9, 157, 105, -84, -18, 163, 9, 17, -20,
	162, -9, -10, -11, -12, -13, 127, -21, 97, -3,
	-90, -91, -21, -21, 163, 163, 163, 163, 9, 163,
	9, -3, 17, 163, 9, -21, -21, -21, -21, -21,
	-21, -21, -21, -21, -21, -21, -21, -21, -21, -21,
	-21, -21, -21, -21, -21, -21, -21, -21, -21, -21,
	-21, -48, -21, 31, -21, -101, -28, -29, -21, -88,
	-112, 163, 163, 86, 164, 86, -30, 131, 161, -117,
	131, 164, -9, 164, 163, 31, -21, 37, -21, -21,
	-21, -21, -21, -21, -21, -21, -21, -21, -21, -21,
	-21, -22, -21, -56, 86, 161, -49, -107, 166, 9,
	15, -29, 164, -21, -107, 165, -22, 160, -65, -65,
	93, 69, 133, -21, 87, -29, 15, -22, -55, -6,
	-49, 161, 86, -5, -4, 76, 77, 78, 79, 80,
	81, 4, 5, 6, 7, 8, 10, 11, 12, 56,
	70, 71, 82, 83, 73, 74, 75, 95, 96, 97,
	98, 99, 100, 101, 102, 103, 104, 105, 116, 117,
	118, 119, 120, 121, 122, 123, 124, 125, 126, 111,
	112, 113, 114, 115, 13, 14, 134, 106, 107, 108,
	109, 110, 135, 136, 131, 132, 154, 129, 130, 128,
	137, 138, 140, 139, 141, 142, 156, 155, -55, -6,
	-49, -108, -109, 153, 69, 161, 133, 153, -108, -105,
	-74, -21, -21, -21, 150, 150, 159, -21, 165, -110,
	-33, -21, 158, -107, 86, 163, -111, 162, 163, 163,
	157, -84, -17, 157, -84, 161, 86, 157, -19, -21,
	164, 165, 164, 163, 9, 165, -35, -36, 164, -21,
	-44, 31, -21, -125, 9, 105, 165, 162, 164, -113,
	-114, -31, -32, -77, -75, 30, 135, 136, -8, -30,
	-118, 132, -8, -115, 161, -122, -8, -21, -21, -112,
	-21, 165, 166, 162, -109, -21, 165, 17, -73, -21,
	37, 134, -107, 165, 166, 165, -66, 86, 89, 50,
	88, 86, 162, 162, 69, 162, -21, 166, -109, -21,
	-109, -49, -22, -21, -56, -49, -30, -125, 9, 165,
	165, 162, -125, 9, -21, 165, 162, 161, 157, -94,
	-15, -18, -80, 161, 165, -23, -9, 31, -21, -90,
	-21, -97, 161, 31, -113, -21, 165, -28, -81, -29,
	37, 134, 69, -26, -9, 31, -86, -113, 165, 9,
	-1, 37, -75, -118, 161, -122, 162, -37, -120, -119,
	120, -121, 123, -79, 81, 80, 79, 76, 77, 78,
	-115, 9, 165, 165, 75, -9, 31, 162, 17, -21,
	-29, 164, 165, 166, 89, -21, 162, 166, 162, -118,
	165, -74, 165, -33, -93, -18, 161, -125, 9, -18,
	-94, 163, -112, 165, 163, -98, 163, -98, 163, 165,
	163, 165, 15, -29, 164, -107, -112, -27, 117, 118,
	165, -104, 120, -31, -2, 158, 161, -115, -95, -43,
	88, 114, 113, -122, -79, 162, -8, -9, 31, 163,
	-112, -21, -107, 166, 161, -125, 9, -93, 162, -15,
	-125, 98, 163, -90, 162, 108, 109, -98, 107, -98,
	-104, -25, -9, 31, -81, -29, -107, 166, 104, 164,
	161, -76, 31, -76, 164, 88, -115, 162, 163, 9,
	17, -87, -70, -6, -3, -100, 163, 161, -112, 165,
	162, -115, 162, -18, -125, 162, 163, 165, -21, -126,
	31, 163, 162, 163, 107, -76, -112, 165, 165, 163,
	-85, -8, -112, 161, -77, 161, -103, -53, 88, 37,
	17, 162, -43, -21, 163, 9, 17, -6, 162, -99,
	-38, -39, -40, -41, -42, -8, -6, 154, 86, 162,
	162, -24, -9, 31, -126, -112, 163, 161, 102, -25,
	88, 35, 162, -112, -112, 165, 9, 88, -21, -70,
	-21, 164, 162, -38, 163, 163, 121, 105, 153, -112,
	-112, -112, 163, 165, -8, 162, 162, -53, -113, -122,
	86, -4, -79, -6, 100, 162, 161, 165, -6, 163,
	-112, -76, 162, -116, 163, 161, -112, 162,
}
var yyDef = [...]int{

	79, -2, 1, 78, 85, 86, 87, 88, 89, 0,
	0, 0, 0, 122, 130, 131, 0, 0, 0, 0,
	416, 416, 416, 0, 381, 0, 142, 0, 0, 0,
	0, 148, 0, 0, 0, 80, 369, 0, 0, 0,
	0, 204, 0, -2, 415, 169, 0, -2, 432, 418,
	0, 452, 0, 0, 0, 0, 0, 0, 0, 0,
	346, 350, 0, 0, 0, 0, 0, 0, 0, 385,
	0, 360, 387, 0, 363, 0, 171, 172, 425, 410,
	430, 0, 0, -2, 0, 0, 0, 0, 0, 0,
	0, 0, 395, 396, 397, 398, 399, 400, 401, 402,
	403, 404, 0, 0, 434, 0, -2, 0, 0, 394,
	82, 0, 0, 0, 0, 79, 80, 0, 0, 0,
	115, 0, 99, 100, 112, 117, 0, 120, 0, 0,
	0, 0, 369, 0, 286, 0, 0, 417, 381, 0,
	0, 0, 232, 233, 0, 369, 235, 236, 0, 284,
	285, 143, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 122, 0, 0, 152, 0, 370,
	0, 170, 175, 0, 177, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 310, 312, 0, 416, 0, 0, 452,
	0, 451, 455, 453, 457, 0, 0, 297, -2, 0,
	0, -2, 381, 452, -2, 331, 332, 333, 334, 0,
	351, 352, 353, 354, 355, 356, 357, 358, 416, 359,
	0, 388, 389, 464, 466, 0, 0, 362, 364, 366,
	416, 0, 0, 390, 292, 383, 384, 390, 382, 439,
	0, 0, 479, 480, 0, 482, 483, 0, 406, 0,
	0, 0, 0, 436, 377, 0, 380, 452, 0, 84,
	0, 83, 91, 79, 0, 94, 0, 0, 115, 0,
	96, 0, 0, 0, 115, 118, 98, 0, 0, 121,
	129, 123, 124, 125, 126, 127, 0, 0, 0, 0,
	0, 287, 289, 0, 136, 137, 138, 139, 0, 140,
	0, 0, 0, 141, 0, 314, 315, 316, 317, 318,
	319, 320, 321, 322, 323, 324, 325, 326, 327, 328,
	329, 330, -2, -2, -2, -2, -2, -2, -2, -2,
	-2, 344, 0, 0, 349, 105, 159, -2, 0, 0,
	0, 150, 151, 0, -2, 175, 179, 0, 239, 0,
	0, 0, 205, 0, 208, 122, 295, 0, 298, 299,
	300, 301, 302, 303, 304, 305, 306, 307, 308, 309,
	0, 0, 0, 433, 448, 0, 450, 0, 393, 452,
	0, -2, 452, 0, 0, -2, 0, 361, 465, 462,
	463, 0, 0, 0, 0, 419, 0, 0, 0, -2,
	-2, 0, 76, 77, 69, 70, 71, 72, 73, 74,
	75, 2, 3, 4, 5, 6, 7, 8, 9, 10,
	11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
	21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
	31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
	41, 42, 43, 44, 45, 46, 47, 48, 49, 50,
	51, 52, 53, 54, 55, 56, 57, 58, 59, 60,
	61, 62, 63, 64, 65, 66, 67, 68, 0, -2,
	-2, 291, 391, 0, 416, 0, 0, 0, 175, 105,
	484, 486, 0, 0, 405, 408, 407, 0, 225, 105,
	227, 229, 0, 0, 81, 90, 0, 93, 95, 97,
	0, 115, 111, 0, 115, 0, 116, 0, 119, 282,
	0, 0, 0, 286, 0, 0, 231, 234, -2, 237,
	283, 0, 348, 0, 106, 0, 0, 153, -2, 0,
	210, 212, 163, 217, 218, 0, 220, 221, 222, 179,
	0, 0, 176, 265, 239, 178, 244, 0, 0, 0,
	296, 0, 426, 428, 429, 0, 0, 0, 454, 456,
	0, 0, 0, -2, 393, 386, 0, 473, 474, 0,
	476, 468, 469, 470, 0, 472, 365, 427, 378, 0,
	379, 443, 0, 0, 442, 444, 179, 0, 106, 478,
	481, 435, 0, 106, 230, 392, 92, 0, 0, 105,
	108, 113, 0, 0, 0, 132, 200, 122, 0, 0,
	288, 135, 195, 195, 0, 347, 0, 160, 0, -2,
	0, 0, 452, 147, 189, 122, 157, 0, 371, 216,
	165, 164, 219, 0, 239, 180, 173, 238, 0, 0,
	0, -2, 264, 267, 269, 270, 271, 272, 273, 274,
	265, 0, 0, 0, 0, 202, 122, 449, 0, 294,
	-2, 452, 461, 467, 475, 0, 446, 440, 441, 0,
	477, 485, 226, 228, 105, 110, 0, 0, 106, 114,
	105, 128, 0, 0, 286, 0, 195, 0, 195, 371,
	144, 0, 0, -2, 452, 0, 0, 149, 0, 0,
	223, 223, 0, 213, 0, 166, 239, 265, 0, 276,
	277, 0, 369, 0, 268, 174, 245, 203, 122, 209,
	206, 293, 0, 0, 239, 0, 106, 105, 103, 107,
	0, 0, 133, 0, 191, 0, 0, 0, 0, 0,
	223, 145, 187, 122, 0, -2, 0, -2, 0, 0,
	122, 0, 0, 0, 0, 214, 265, 168, 240, 0,
	0, 0, 280, 0, 0, 242, 246, 0, 207, 460,
	471, 265, 101, 109, 0, 104, 201, 0, 0, 122,
	198, 199, 192, 193, 0, 0, 0, 0, 183, 190,
	0, 155, 0, 122, 224, 122, 0, 374, 375, 0,
	0, 167, 275, 278, 241, 0, 0, 0, 247, 0,
	249, 0, 0, 259, 0, 0, 258, 57, -2, 290,
	102, 134, 185, 122, 122, 197, 194, 122, 0, 146,
	0, 0, 158, 0, 0, 372, 0, 376, 215, 279,
	281, -2, 248, 250, 251, 252, 0, 0, 0, 0,
	196, 0, 188, 0, 156, 162, 367, 373, 0, 253,
	254, 255, 257, 260, 0, 368, 122, 223, 256, 186,
	0, 0, 154, 243, 261, 122, 0, 262,
}
var yyTok1 = [...]int{

	1, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 55, 159, 3, 167, 54, 37, 3,
	164, 165, 52, 49, 9, 50, 51, 53, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 31, 163,
	43, 17, 45, 30, 67, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 69, 3, 166, 36, 3, 160, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 161, 35, 162, 57,
}
var yyTok2 = [...]int{

	2, 3, 4, 5, 6, 7, 8, 10, 11, 12,
	13, 14, 15, 16, 18, 19, 20, 21, 22, 23,
	24, 25, 26, 27, 28, 29, 32, 33, 34, 38,
	39, 40, 41, 42, 44, 46, 47, 48, 56, 58,
	59, 60, 61, 62, 63, 64, 65, 66, 68, 70,
	71, 72, 73, 74, 75, 76, 77, 78, 79, 80,
	81, 82, 83, 84, 85, 86, 87, 88, 89, 90,
	91, 92, 93, 94, 95, 96, 97, 98, 99, 100,
	101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
	111, 112, 113, 114, 115, 116, 117, 118, 119, 120,
	121, 122, 123, 124, 125, 126, 127, 128, 129, 130,
	131, 132, 133, 134, 135, 136, 137, 138, 139, 140,
	141, 142, 143, 144, 145, 146, 147, 148, 149, 150,
	151, 152, 153, 154, 155, 156, 157, 158,
}
var yyTok3 = [...]int{
	0,
}

var yyErrorMessages = [...]struct {
	state int
	token int
	msg   string
}{}

//line yaccpar:1

/*	parser for yacc output	*/

var (
	yyDebug        = 0
	yyErrorVerbose = false
)

type yyLexer interface {
	Lex(lval *yySymType) int
	Error(s string)
}

type yyParser interface {
	Parse(yyLexer) int
	Lookahead() int
}

type yyParserImpl struct {
	lval  yySymType
	stack [yyInitialStackSize]yySymType
	char  int
}

func (p *yyParserImpl) Lookahead() int {
	return p.char
}

func yyNewParser() yyParser {
	return &yyParserImpl{}
}

const yyFlag = -1000

func yyTokname(c int) string {
	if c >= 1 && c-1 < len(yyToknames) {
		if yyToknames[c-1] != "" {
			return yyToknames[c-1]
		}
	}
	return __yyfmt__.Sprintf("tok-%v", c)
}

func yyStatname(s int) string {
	if s >= 0 && s < len(yyStatenames) {
		if yyStatenames[s] != "" {
			return yyStatenames[s]
		}
	}
	return __yyfmt__.Sprintf("state-%v", s)
}

func yyErrorMessage(state, lookAhead int) string {
	const TOKSTART = 4

	if !yyErrorVerbose {
		return "syntax error"
	}

	for _, e := range yyErrorMessages {
		if e.state == state && e.token == lookAhead {
			return "syntax error: " + e.msg
		}
	}

	res := "syntax error: unexpected " + yyTokname(lookAhead)

	// To match Bison, suggest at most four expected tokens.
	expected := make([]int, 0, 4)

	// Look for shiftable tokens.
	base := yyPact[state]
	for tok := TOKSTART; tok-1 < len(yyToknames); tok++ {
		if n := base + tok; n >= 0 && n < yyLast && yyChk[yyAct[n]] == tok {
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}
	}

	if yyDef[state] == -2 {
		i := 0
		for yyExca[i] != -1 || yyExca[i+1] != state {
			i += 2
		}

		// Look for tokens that we accept or reduce.
		for i += 2; yyExca[i] >= 0; i += 2 {
			tok := yyExca[i]
			if tok < TOKSTART || yyExca[i+1] == 0 {
				continue
			}
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}

		// If the default action is to accept or reduce, give up.
		if yyExca[i+1] != 0 {
			return res
		}
	}

	for i, tok := range expected {
		if i == 0 {
			res += ", expecting "
		} else {
			res += " or "
		}
		res += yyTokname(tok)
	}
	return res
}

func yylex1(lex yyLexer, lval *yySymType) (char, token int) {
	token = 0
	char = lex.Lex(lval)
	if char <= 0 {
		token = yyTok1[0]
		goto out
	}
	if char < len(yyTok1) {
		token = yyTok1[char]
		goto out
	}
	if char >= yyPrivate {
		if char < yyPrivate+len(yyTok2) {
			token = yyTok2[char-yyPrivate]
			goto out
		}
	}
	for i := 0; i < len(yyTok3); i += 2 {
		token = yyTok3[i+0]
		if token == char {
			token = yyTok3[i+1]
			goto out
		}
	}

out:
	if token == 0 {
		token = yyTok2[1] /* unknown char */
	}
	if yyDebug >= 3 {
		__yyfmt__.Printf("lex %s(%d)\n", yyTokname(token), uint(char))
	}
	return char, token
}

func yyParse(yylex yyLexer) int {
	return yyNewParser().Parse(yylex)
}

func (yyrcvr *yyParserImpl) Parse(yylex yyLexer) int {
	var yyn int
	var yyVAL yySymType
	var yyDollar []yySymType
	_ = yyDollar // silence set and not used
	yyS := yyrcvr.stack[:]

	Nerrs := 0   /* number of errors */
	Errflag := 0 /* error recovery flag */
	yystate := 0
	yyrcvr.char = -1
	yytoken := -1 // yyrcvr.char translated into internal numbering
	defer func() {
		// Make sure we report no lookahead when not parsing.
		yystate = -1
		yyrcvr.char = -1
		yytoken = -1
	}()
	yyp := -1
	goto yystack

ret0:
	return 0

ret1:
	return 1

yystack:
	/* put a state and value onto the stack */
	if yyDebug >= 4 {
		__yyfmt__.Printf("char %v in %v\n", yyTokname(yytoken), yyStatname(yystate))
	}

	yyp++
	if yyp >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyS[yyp] = yyVAL
	yyS[yyp].yys = yystate

yynewstate:
	yyn = yyPact[yystate]
	if yyn <= yyFlag {
		goto yydefault /* simple state */
	}
	if yyrcvr.char < 0 {
		yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
	}
	yyn += yytoken
	if yyn < 0 || yyn >= yyLast {
		goto yydefault
	}
	yyn = yyAct[yyn]
	if yyChk[yyn] == yytoken { /* valid shift */
		yyrcvr.char = -1
		yytoken = -1
		yyVAL = yyrcvr.lval
		yystate = yyn
		if Errflag > 0 {
			Errflag--
		}
		goto yystack
	}

yydefault:
	/* default state action */
	yyn = yyDef[yystate]
	if yyn == -2 {
		if yyrcvr.char < 0 {
			yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
		}

		/* look through exception table */
		xi := 0
		for {
			if yyExca[xi+0] == -1 && yyExca[xi+1] == yystate {
				break
			}
			xi += 2
		}
		for xi += 2; ; xi += 2 {
			yyn = yyExca[xi+0]
			if yyn < 0 || yyn == yytoken {
				break
			}
		}
		yyn = yyExca[xi+1]
		if yyn < 0 {
			goto ret0
		}
	}
	if yyn == 0 {
		/* error ... attempt to resume parsing */
		switch Errflag {
		case 0: /* brand new error */
			yylex.Error(yyErrorMessage(yystate, yytoken))
			Nerrs++
			if yyDebug >= 1 {
				__yyfmt__.Printf("%s", yyStatname(yystate))
				__yyfmt__.Printf(" saw %s\n", yyTokname(yytoken))
			}
			fallthrough

		case 1, 2: /* incompletely recovered error ... try again */
			Errflag = 3

			/* find a state where "error" is a legal shift action */
			for yyp >= 0 {
				yyn = yyPact[yyS[yyp].yys] + yyErrCode
				if yyn >= 0 && yyn < yyLast {
					yystate = yyAct[yyn] /* simulate a shift of "error" */
					if yyChk[yystate] == yyErrCode {
						goto yystack
					}
				}

				/* the current p has no shift on "error", pop stack */
				if yyDebug >= 2 {
					__yyfmt__.Printf("error recovery pops state %d\n", yyS[yyp].yys)
				}
				yyp--
			}
			/* there is no state on the stack with an error shift ... abort */
			goto ret1

		case 3: /* no shift yet; clobber input char */
			if yyDebug >= 2 {
				__yyfmt__.Printf("error recovery discards %s\n", yyTokname(yytoken))
			}
			if yytoken == yyEofCode {
				goto ret1
			}
			yyrcvr.char = -1
			yytoken = -1
			goto yynewstate /* try again in the same state */
		}
	}

	/* reduction by production yyn */
	if yyDebug >= 2 {
		__yyfmt__.Printf("reduce %v in:\n\t%v\n", yyn, yyStatname(yystate))
	}

	yynt := yyn
	yypt := yyp
	_ = yypt // guard against "declared and not used"

	yyp -= yyR2[yyn]
	// yyp is now the index of $0. Perform the default action. Iff the
	// reduced production is ε, $1 is possibly out of range.
	if yyp+1 >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyVAL = yyS[yyp+1]

	/* consult goto table to find next state */
	yyn = yyR1[yyn]
	yyg := yyPgo[yyn]
	yyj := yyg + yyS[yyp].yys + 1

	if yyj >= yyLast {
		yystate = yyAct[yyg]
	} else {
		yystate = yyAct[yyj]
		if yyChk[yystate] != -yyn {
			yystate = yyAct[yyg]
		}
	}
	// dummy call; replaced with literal code
	switch yynt {

	case 1:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:243
		{
			rootnode = stmt.NewStmtList(yyDollar[1].list)
		}
	case 2:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 3:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 4:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 5:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 6:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 7:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 8:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 9:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:247
		{
			yyVAL.token = yyDollar[1].token
		}
	case 10:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 11:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 12:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 13:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 14:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 15:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 16:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 17:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 18:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 19:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 20:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 21:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:248
		{
			yyVAL.token = yyDollar[1].token
		}
	case 22:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 23:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 24:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 25:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 26:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 27:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 28:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 29:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 30:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 31:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:249
		{
			yyVAL.token = yyDollar[1].token
		}
	case 32:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 33:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 34:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 35:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 36:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 37:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 38:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 39:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 40:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 41:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:250
		{
			yyVAL.token = yyDollar[1].token
		}
	case 42:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 43:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 44:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 45:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 46:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 47:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 48:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 49:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 50:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 51:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 52:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:251
		{
			yyVAL.token = yyDollar[1].token
		}
	case 53:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 54:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 55:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 56:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 57:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 58:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 59:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 60:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:252
		{
			yyVAL.token = yyDollar[1].token
		}
	case 61:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 62:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 63:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 64:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 65:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 66:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 67:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 68:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:253
		{
			yyVAL.token = yyDollar[1].token
		}
	case 69:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:257
		{
			yyVAL.token = yyDollar[1].token
		}
	case 70:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:258
		{
			yyVAL.token = yyDollar[1].token
		}
	case 71:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:258
		{
			yyVAL.token = yyDollar[1].token
		}
	case 72:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:258
		{
			yyVAL.token = yyDollar[1].token
		}
	case 73:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:258
		{
			yyVAL.token = yyDollar[1].token
		}
	case 74:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:258
		{
			yyVAL.token = yyDollar[1].token
		}
	case 75:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:258
		{
			yyVAL.token = yyDollar[1].token
		}
	case 76:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:262
		{
			yyVAL.token = yyDollar[1].token
		}
	case 77:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:263
		{
			yyVAL.token = yyDollar[1].token
		}
	case 78:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:267
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 79:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:268
		{
			yyVAL.list = []node.Node{}
		}
	case 80:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:272
		{
			yyVAL.list = []node.Node{name.NewNamePart(yyDollar[1].token)}
		}
	case 81:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:273
		{
			yyVAL.list = append(yyDollar[1].list, name.NewNamePart(yyDollar[3].token))
		}
	case 82:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:277
		{
			yyVAL.node = name.NewName(yyDollar[1].list)
		}
	case 83:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:278
		{
			yyVAL.node = name.NewRelative(yyDollar[3].list)
		}
	case 84:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:279
		{
			yyVAL.node = name.NewFullyQualified(yyDollar[2].list)
		}
	case 85:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:283
		{
			yyVAL.node = yyDollar[1].node
		}
	case 86:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:284
		{
			yyVAL.node = yyDollar[1].node
		}
	case 87:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:285
		{
			yyVAL.node = yyDollar[1].node
		}
	case 88:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:286
		{
			yyVAL.node = yyDollar[1].node
		}
	case 89:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:287
		{
			yyVAL.node = yyDollar[1].node
		}
	case 90:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:288
		{
			yyVAL.node = stmt.NewHaltCompiler(yyDollar[1].token)
		}
	case 91:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:289
		{
			yyVAL.node = stmt.NewNamespace(yyDollar[1].token, name.NewName(yyDollar[2].list), nil)
		}
	case 92:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:291
		{
			yyVAL.node = stmt.NewNamespace(yyDollar[1].token, name.NewName(yyDollar[2].list), yyDollar[4].list)
		}
	case 93:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:292
		{
			yyVAL.node = stmt.NewNamespace(yyDollar[1].token, nil, yyDollar[3].list)
		}
	case 94:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:293
		{
			yyVAL.node = yyDollar[2].node.(stmt.GroupUse).SetToken(yyDollar[1].token)
		}
	case 95:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:294
		{
			yyVAL.node = yyDollar[3].node.(stmt.GroupUse).SetToken(yyDollar[1].token).(stmt.GroupUse).SetUseType(yyDollar[2].node)
		}
	case 96:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:295
		{
			yyVAL.node = stmt.NewUseList(yyDollar[1].token, nil, yyDollar[2].list)
		}
	case 97:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:296
		{
			yyVAL.node = stmt.NewUseList(yyDollar[1].token, yyDollar[2].node, yyDollar[3].list)
		}
	case 98:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:297
		{
			yyVAL.node = stmt.NewConstList(yyDollar[1].token, yyDollar[2].list)
		}
	case 99:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:301
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 100:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:302
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 101:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:307
		{
			fmt.Println("group")
			yyVAL.node = stmt.NewGroupUse(nil, nil, name.NewName(yyDollar[1].list), yyDollar[4].list)
		}
	case 102:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line parser/parser.y:312
		{
			yyVAL.node = stmt.NewGroupUse(nil, nil, name.NewName(yyDollar[2].list), yyDollar[5].list)
		}
	case 103:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:319
		{
			fmt.Println("mixed")
			yyVAL.node = stmt.NewGroupUse(nil, nil, name.NewName(yyDollar[1].list), yyDollar[4].list)
		}
	case 104:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line parser/parser.y:324
		{
			yyVAL.node = stmt.NewGroupUse(nil, nil, name.NewName(yyDollar[2].list), yyDollar[5].list)
		}
	case 107:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:336
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 108:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:337
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 109:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:342
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 110:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:343
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 111:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:347
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 112:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:348
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 113:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:352
		{
			yyVAL.node = yyDollar[1].node
		}
	case 114:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:353
		{
			yyVAL.node = yyDollar[2].node.(stmt.Use).SetType(yyDollar[1].node)
		}
	case 115:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:357
		{
			yyVAL.node = stmt.NewUse(nil, name.NewName(yyDollar[1].list), nil)
		}
	case 116:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:358
		{
			yyVAL.node = stmt.NewUse(nil, name.NewName(yyDollar[1].list), yyDollar[3].token)
		}
	case 117:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:362
		{
			yyVAL.node = yyDollar[1].node
		}
	case 118:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:363
		{
			yyVAL.node = yyDollar[2].node
		}
	case 119:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:367
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 120:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:368
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 121:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:372
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 122:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:373
		{
			yyVAL.list = []node.Node{}
		}
	case 123:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:377
		{
			yyVAL.node = yyDollar[1].node
		}
	case 124:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:378
		{
			yyVAL.node = yyDollar[1].node
		}
	case 125:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:379
		{
			yyVAL.node = yyDollar[1].node
		}
	case 126:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:380
		{
			yyVAL.node = yyDollar[1].node
		}
	case 127:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:381
		{
			yyVAL.node = yyDollar[1].node
		}
	case 128:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:382
		{
			yyVAL.node = stmt.NewHaltCompiler(yyDollar[1].token)
		}
	case 129:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:385
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)
		}
	case 130:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:386
		{
			yyVAL.node = yyDollar[1].node
		}
	case 131:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:387
		{
			yyVAL.node = yyDollar[1].node
		}
	case 132:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:389
		{
			yyVAL.node = stmt.NewWhile(yyDollar[1].token, yyDollar[3].node, yyDollar[5].node)
		}
	case 133:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line parser/parser.y:390
		{
			yyVAL.node = stmt.NewDo(yyDollar[1].token, yyDollar[2].node, yyDollar[5].node)
		}
	case 134:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line parser/parser.y:392
		{
			yyVAL.node = stmt.NewFor(yyDollar[1].token, yyDollar[3].list, yyDollar[5].list, yyDollar[7].list, yyDollar[9].node)
		}
	case 135:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:393
		{
			yyVAL.node = stmt.NewSwitch(yyDollar[1].token, yyDollar[3].node, yyDollar[5].list)
		}
	case 136:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:394
		{
			yyVAL.node = stmt.NewBreak(yyDollar[1].token, yyDollar[2].node)
		}
	case 137:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:395
		{
			yyVAL.node = stmt.NewContinue(yyDollar[1].token, yyDollar[2].node)
		}
	case 138:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:396
		{
			yyVAL.node = stmt.NewReturn(yyDollar[1].token, yyDollar[2].node)
		}
	case 139:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:397
		{
			yyVAL.node = stmt.NewGlobal(yyDollar[1].token, yyDollar[2].list)
		}
	case 140:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:398
		{
			yyVAL.node = stmt.NewStatic(yyDollar[1].token, yyDollar[2].list)
		}
	case 141:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:399
		{
			yyVAL.node = stmt.NewEcho(yyDollar[1].token, yyDollar[2].list)
		}
	case 142:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:400
		{
			yyVAL.node = stmt.NewInlineHtml(yyDollar[1].token)
		}
	case 143:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:401
		{
			yyVAL.node = stmt.NewExpression(yyDollar[1].node)
		}
	case 144:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:403
		{
			yyVAL.node = stmt.NewUnset(yyDollar[1].token, yyDollar[3].list)
		}
	case 145:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line parser/parser.y:405
		{
			yyVAL.node = stmt.NewForeach(yyDollar[1].token, yyDollar[3].node, nil, yyDollar[5].foreachVariable.node, yyDollar[7].node, yyDollar[5].foreachVariable.byRef)
		}
	case 146:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line parser/parser.y:407
		{
			yyVAL.node = stmt.NewForeach(yyDollar[1].token, yyDollar[3].node, yyDollar[5].node, yyDollar[7].foreachVariable.node, yyDollar[9].node, yyDollar[7].foreachVariable.byRef)
		}
	case 147:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:408
		{
			yyVAL.node = stmt.NewDeclare(yyDollar[1].token, yyDollar[3].list, yyDollar[5].node)
		}
	case 148:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:409
		{
			yyVAL.node = stmt.NewNop(yyDollar[1].token)
		}
	case 149:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:411
		{
			yyVAL.node = stmt.NewTry(yyDollar[1].token, yyDollar[3].list, yyDollar[5].list, yyDollar[6].node)
		}
	case 150:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:414
		{
			yyVAL.node = stmt.NewThrow(yyDollar[1].token, yyDollar[2].node)
		}
	case 151:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:415
		{
			yyVAL.node = stmt.NewGoto(yyDollar[1].token, yyDollar[2].token)
		}
	case 152:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:416
		{
			yyVAL.node = stmt.NewLabel(yyDollar[1].token)
		}
	case 153:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:419
		{
			yyVAL.list = []node.Node{}
		}
	case 154:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line parser/parser.y:421
		{
			yyVAL.list = append(yyDollar[1].list, stmt.NewCatch(yyDollar[2].token, yyDollar[4].list, expr.NewVariable(node.NewIdentifier(yyDollar[5].token)), yyDollar[8].list))
		}
	case 155:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:424
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 156:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:425
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 157:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:429
		{
			yyVAL.node = nil
		}
	case 158:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:430
		{
			yyVAL.node = stmt.NewFinally(yyDollar[1].token, yyDollar[3].list)
		}
	case 159:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:434
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 160:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:435
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 161:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:439
		{
			yyVAL.node = yyDollar[1].node
		}
	case 162:
		yyDollar = yyS[yypt-10 : yypt+1]
		//line parser/parser.y:444
		{
			yyVAL.node = stmt.NewFunction(yyDollar[3].token, yyDollar[2].boolean, yyDollar[5].list, yyDollar[7].node, yyDollar[9].list)
		}
	case 163:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:450
		{
			yyVAL.boolean = false
		}
	case 164:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:451
		{
			yyVAL.boolean = true
		}
	case 165:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:455
		{
			yyVAL.boolean = false
		}
	case 166:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:456
		{
			yyVAL.boolean = true
		}
	case 167:
		yyDollar = yyS[yypt-8 : yypt+1]
		//line parser/parser.y:461
		{
			yyVAL.node = stmt.NewClass(yyDollar[3].token, yyDollar[1].list, nil, yyDollar[4].node, yyDollar[5].list, yyDollar[7].list)
		}
	case 168:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line parser/parser.y:463
		{
			yyVAL.node = stmt.NewClass(yyDollar[2].token, nil, nil, yyDollar[3].node, yyDollar[4].list, yyDollar[6].list)
		}
	case 169:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:467
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 170:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:468
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 171:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:472
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 172:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:473
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 173:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:477
		{
			yyVAL.node = stmt.NewTrait(yyDollar[2].token, yyDollar[4].list)
		}
	case 174:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:482
		{
			yyVAL.node = stmt.NewInterface(yyDollar[1].token, yyDollar[2].token, yyDollar[3].list, yyDollar[5].list)
		}
	case 175:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:486
		{
			yyVAL.node = nil
		}
	case 176:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:487
		{
			yyVAL.node = yyDollar[2].node
		}
	case 177:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:491
		{
			yyVAL.list = nil
		}
	case 178:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:492
		{
			yyVAL.list = yyDollar[2].list
		}
	case 179:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:496
		{
			yyVAL.list = nil
		}
	case 180:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:497
		{
			yyVAL.list = yyDollar[2].list
		}
	case 181:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:501
		{
			yyVAL.foreachVariable = foreachVariable{yyDollar[1].node, false}
		}
	case 182:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:502
		{
			yyVAL.foreachVariable = foreachVariable{yyDollar[2].node, true}
		}
	case 183:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:503
		{
			yyVAL.foreachVariable = foreachVariable{expr.NewList(yyDollar[3].list), false}
		}
	case 184:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:504
		{
			yyVAL.foreachVariable = foreachVariable{expr.NewShortList(yyDollar[2].list), false}
		}
	case 185:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:508
		{
			yyVAL.node = yyDollar[1].node
		}
	case 186:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:509
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)
		}
	case 187:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:513
		{
			yyVAL.node = yyDollar[1].node
		}
	case 188:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:514
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)
		}
	case 189:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:518
		{
			yyVAL.node = yyDollar[1].node
		}
	case 190:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:519
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)
		}
	case 191:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:523
		{
			yyVAL.list = yyDollar[2].list
		}
	case 192:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:524
		{
			yyVAL.list = yyDollar[3].list
		}
	case 193:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:525
		{
			yyVAL.list = yyDollar[2].list
		}
	case 194:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:526
		{
			yyVAL.list = yyDollar[3].list
		}
	case 195:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:530
		{
			yyVAL.list = []node.Node{}
		}
	case 196:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:532
		{
			yyVAL.list = append(yyDollar[1].list, stmt.NewCase(yyDollar[2].token, yyDollar[3].node, yyDollar[5].list))
		}
	case 197:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:536
		{
			yyVAL.list = append(yyDollar[1].list, stmt.NewDefault(yyDollar[2].token, yyDollar[4].list))
		}
	case 200:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:547
		{
			yyVAL.node = yyDollar[1].node
		}
	case 201:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:548
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)
		}
	case 202:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:552
		{
			yyVAL.node = stmt.NewIf(yyDollar[1].token, yyDollar[3].node, yyDollar[5].node)
		}
	case 203:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:554
		{
			_elseIf := stmt.NewElseIf(yyDollar[2].token, yyDollar[4].node, yyDollar[6].node)
			yyVAL.node = yyDollar[1].node.(stmt.If).AddElseIf(_elseIf)
		}
	case 204:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:561
		{
			yyVAL.node = yyDollar[1].node
		}
	case 205:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:563
		{
			_else := stmt.NewElse(yyDollar[2].token, yyDollar[3].node)
			yyVAL.node = yyDollar[1].node.(stmt.If).SetElse(_else)
		}
	case 206:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:571
		{
			yyVAL.node = stmt.NewAltIf(yyDollar[1].token, yyDollar[3].node, stmt.NewStmtList(yyDollar[6].list))
		}
	case 207:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line parser/parser.y:575
		{
			_elseIf := stmt.NewAltElseIf(yyDollar[2].token, yyDollar[4].node, stmt.NewStmtList(yyDollar[7].list))
			yyVAL.node = yyDollar[1].node.(stmt.AltIf).AddElseIf(_elseIf)
		}
	case 208:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:582
		{
			yyVAL.node = yyDollar[1].node
		}
	case 209:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:584
		{
			_else := stmt.NewAltElse(yyDollar[2].token, stmt.NewStmtList(yyDollar[4].list))
			yyVAL.node = yyDollar[1].node.(stmt.AltIf).SetElse(_else)
		}
	case 210:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:591
		{
			yyVAL.list = yyDollar[1].list
		}
	case 211:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:592
		{
			yyVAL.list = nil
		}
	case 212:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:596
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 213:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:597
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 214:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:602
		{
			yyVAL.node = node.NewParameter(yyDollar[1].node, expr.NewVariable(node.NewIdentifier(yyDollar[4].token)), nil, yyDollar[2].boolean, yyDollar[3].boolean)
		}
	case 215:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:606
		{
			yyVAL.node = node.NewParameter(yyDollar[1].node, expr.NewVariable(node.NewIdentifier(yyDollar[4].token)), yyDollar[6].node, yyDollar[2].boolean, yyDollar[3].boolean)
		}
	case 216:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:612
		{
			yyVAL.node = nil
		}
	case 217:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:613
		{
			yyVAL.node = yyDollar[1].node
		}
	case 218:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:617
		{
			yyVAL.node = yyDollar[1].node
		}
	case 219:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:618
		{
			yyVAL.node = node.NewNullable(yyDollar[2].node)
		}
	case 220:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:622
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 221:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:623
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 222:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:624
		{
			yyVAL.node = yyDollar[1].node
		}
	case 223:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:628
		{
			yyVAL.node = nil
		}
	case 224:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:629
		{
			yyVAL.node = yyDollar[2].node
		}
	case 225:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:633
		{
			yyVAL.list = []node.Node{}
		}
	case 226:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:634
		{
			yyVAL.list = yyDollar[2].list
		}
	case 227:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:638
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 228:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:639
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 229:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:643
		{
			yyVAL.node = node.NewArgument(yyDollar[1].node, false)
		}
	case 230:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:644
		{
			yyVAL.node = node.NewArgument(yyDollar[2].node, true)
		}
	case 231:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:648
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 232:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:649
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 233:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:653
		{
			yyVAL.node = yyDollar[1].node
		}
	case 234:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:657
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 235:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:658
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 236:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:662
		{
			yyVAL.node = stmt.NewStaticVar(yyDollar[1].token, nil)
		}
	case 237:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:663
		{
			yyVAL.node = stmt.NewStaticVar(yyDollar[1].token, yyDollar[3].node)
		}
	case 238:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:667
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 239:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:668
		{
			yyVAL.list = []node.Node{}
		}
	case 240:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:672
		{
			yyVAL.node = stmt.NewPropertyList(yyDollar[1].list, yyDollar[2].list)
		}
	case 241:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:673
		{
			yyVAL.node = stmt.NewClassConstList(yyDollar[2].token, yyDollar[1].list, yyDollar[3].list)
		}
	case 242:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:674
		{
			yyVAL.node = stmt.NewTraitUse(yyDollar[1].token, yyDollar[2].list, yyDollar[3].list)
		}
	case 243:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line parser/parser.y:676
		{
			yyVAL.node = stmt.NewClassMethod(yyDollar[4].token, yyDollar[1].list, yyDollar[3].boolean, yyDollar[6].list, yyDollar[8].node, yyDollar[9].list)
		}
	case 244:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:682
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 245:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:683
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 246:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:687
		{
			yyVAL.list = nil
		}
	case 247:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:688
		{
			yyVAL.list = nil
		}
	case 248:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:689
		{
			yyVAL.list = yyDollar[2].list
		}
	case 249:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:693
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 250:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:694
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 251:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:698
		{
			yyVAL.node = yyDollar[1].node
		}
	case 252:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:699
		{
			yyVAL.node = yyDollar[1].node
		}
	case 253:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:704
		{
			yyVAL.node = stmt.NewTraitUsePrecedence(yyDollar[1].node, name.NewName(yyDollar[3].list))
		}
	case 254:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:710
		{
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, yyDollar[3].token)
		}
	case 255:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:712
		{
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, yyDollar[3].token)
		}
	case 256:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:714
		{
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, yyDollar[4].token)
		}
	case 257:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:715
		{
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, nil)
		}
	case 258:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:719
		{
			yyVAL.node = stmt.NewTraitMethodRef(nil, yyDollar[1].token)
		}
	case 259:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:720
		{
			yyVAL.node = yyDollar[1].node
		}
	case 260:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:724
		{
			yyVAL.node = stmt.NewTraitMethodRef(yyDollar[1].node, yyDollar[3].token)
		}
	case 261:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:728
		{
			yyVAL.list = nil
		}
	case 262:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:729
		{
			yyVAL.list = yyDollar[2].list
		}
	case 263:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:733
		{
			yyVAL.list = yyDollar[1].list
		}
	case 264:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:734
		{
			yyVAL.list = []node.Node{node.NewIdentifier(yyDollar[1].token)}
		}
	case 265:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:738
		{
			yyVAL.list = nil
		}
	case 266:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:739
		{
			yyVAL.list = yyDollar[1].list
		}
	case 267:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:743
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 268:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:744
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 269:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:748
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 270:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:749
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 271:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:750
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 272:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:751
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 273:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:752
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 274:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:753
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 275:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:757
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 276:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:758
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 277:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:762
		{
			yyVAL.node = stmt.NewProperty(yyDollar[1].token, nil)
		}
	case 278:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:763
		{
			yyVAL.node = stmt.NewProperty(yyDollar[1].token, yyDollar[3].node)
		}
	case 279:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:767
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 280:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:768
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 281:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:772
		{
			yyVAL.node = stmt.NewConstant(yyDollar[1].token, yyDollar[3].node)
		}
	case 282:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:776
		{
			yyVAL.node = stmt.NewConstant(yyDollar[1].token, yyDollar[3].node)
		}
	case 283:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:780
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 284:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:781
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 285:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:785
		{
			yyVAL.node = yyDollar[1].node
		}
	case 286:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:789
		{
			yyVAL.list = nil
		}
	case 287:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:790
		{
			yyVAL.list = yyDollar[1].list
		}
	case 288:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:793
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 289:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:794
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 290:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line parser/parser.y:799
		{
			{
				yyVAL.node = stmt.NewClass(yyDollar[1].token, nil, yyDollar[2].list, yyDollar[3].node, yyDollar[4].list, yyDollar[6].list)
			}
		}
	case 291:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:805
		{
			yyVAL.node = expr.NewNew(yyDollar[2].node, yyDollar[3].list)
		}
	case 292:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:806
		{
			yyVAL.node = expr.NewNew(yyDollar[2].node, nil)
		}
	case 293:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:811
		{
			list := expr.NewList(yyDollar[3].list)
			yyVAL.node = assign_op.NewAssign(list, yyDollar[6].node)
		}
	case 294:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:816
		{
			shortList := expr.NewShortList(yyDollar[2].list)
			yyVAL.node = assign_op.NewAssign(shortList, yyDollar[5].node)
		}
	case 295:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:820
		{
			yyVAL.node = assign_op.NewAssign(yyDollar[1].node, yyDollar[3].node)
		}
	case 296:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:821
		{
			yyVAL.node = assign_op.NewAssignRef(yyDollar[1].node, yyDollar[4].node)
		}
	case 297:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:822
		{
			yyVAL.node = expr.NewClone(yyDollar[2].node)
		}
	case 298:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:823
		{
			yyVAL.node = assign_op.NewPlus(yyDollar[1].node, yyDollar[3].node)
		}
	case 299:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:824
		{
			yyVAL.node = assign_op.NewMinus(yyDollar[1].node, yyDollar[3].node)
		}
	case 300:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:825
		{
			yyVAL.node = assign_op.NewMul(yyDollar[1].node, yyDollar[3].node)
		}
	case 301:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:826
		{
			yyVAL.node = assign_op.NewPow(yyDollar[1].node, yyDollar[3].node)
		}
	case 302:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:827
		{
			yyVAL.node = assign_op.NewDiv(yyDollar[1].node, yyDollar[3].node)
		}
	case 303:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:828
		{
			yyVAL.node = assign_op.NewConcat(yyDollar[1].node, yyDollar[3].node)
		}
	case 304:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:829
		{
			yyVAL.node = assign_op.NewMod(yyDollar[1].node, yyDollar[3].node)
		}
	case 305:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:830
		{
			yyVAL.node = assign_op.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)
		}
	case 306:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:831
		{
			yyVAL.node = assign_op.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)
		}
	case 307:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:832
		{
			yyVAL.node = assign_op.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)
		}
	case 308:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:833
		{
			yyVAL.node = assign_op.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)
		}
	case 309:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:834
		{
			yyVAL.node = assign_op.NewShiftRight(yyDollar[1].node, yyDollar[3].node)
		}
	case 310:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:835
		{
			yyVAL.node = expr.NewPostInc(yyDollar[1].node)
		}
	case 311:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:836
		{
			yyVAL.node = expr.NewPreInc(yyDollar[2].node)
		}
	case 312:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:837
		{
			yyVAL.node = expr.NewPostDec(yyDollar[1].node)
		}
	case 313:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:838
		{
			yyVAL.node = expr.NewPreDec(yyDollar[2].node)
		}
	case 314:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:839
		{
			yyVAL.node = binary_op.NewBooleanOr(yyDollar[1].node, yyDollar[3].node)
		}
	case 315:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:840
		{
			yyVAL.node = binary_op.NewBooleanAnd(yyDollar[1].node, yyDollar[3].node)
		}
	case 316:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:841
		{
			yyVAL.node = binary_op.NewLogicalOr(yyDollar[1].node, yyDollar[3].node)
		}
	case 317:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:842
		{
			yyVAL.node = binary_op.NewLogicalAnd(yyDollar[1].node, yyDollar[3].node)
		}
	case 318:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:843
		{
			yyVAL.node = binary_op.NewLogicalXor(yyDollar[1].node, yyDollar[3].node)
		}
	case 319:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:844
		{
			yyVAL.node = binary_op.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)
		}
	case 320:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:845
		{
			yyVAL.node = binary_op.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)
		}
	case 321:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:846
		{
			yyVAL.node = binary_op.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)
		}
	case 322:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:847
		{
			yyVAL.node = binary_op.NewConcat(yyDollar[1].node, yyDollar[3].node)
		}
	case 323:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:848
		{
			yyVAL.node = binary_op.NewPlus(yyDollar[1].node, yyDollar[3].node)
		}
	case 324:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:849
		{
			yyVAL.node = binary_op.NewMinus(yyDollar[1].node, yyDollar[3].node)
		}
	case 325:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:850
		{
			yyVAL.node = binary_op.NewMul(yyDollar[1].node, yyDollar[3].node)
		}
	case 326:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:851
		{
			yyVAL.node = binary_op.NewPow(yyDollar[1].node, yyDollar[3].node)
		}
	case 327:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:852
		{
			yyVAL.node = binary_op.NewDiv(yyDollar[1].node, yyDollar[3].node)
		}
	case 328:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:853
		{
			yyVAL.node = binary_op.NewMod(yyDollar[1].node, yyDollar[3].node)
		}
	case 329:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:854
		{
			yyVAL.node = binary_op.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)
		}
	case 330:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:855
		{
			yyVAL.node = binary_op.NewShiftRight(yyDollar[1].node, yyDollar[3].node)
		}
	case 331:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:856
		{
			yyVAL.node = expr.NewUnaryPlus(yyDollar[2].node)
		}
	case 332:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:857
		{
			yyVAL.node = expr.NewUnaryMinus(yyDollar[2].node)
		}
	case 333:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:858
		{
			yyVAL.node = expr.NewBooleanNot(yyDollar[2].node)
		}
	case 334:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:859
		{
			yyVAL.node = expr.NewBitwiseNot(yyDollar[2].node)
		}
	case 335:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:860
		{
			yyVAL.node = binary_op.NewIdentical(yyDollar[1].node, yyDollar[3].node)
		}
	case 336:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:861
		{
			yyVAL.node = binary_op.NewNotIdentical(yyDollar[1].node, yyDollar[3].node)
		}
	case 337:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:862
		{
			yyVAL.node = binary_op.NewEqual(yyDollar[1].node, yyDollar[3].node)
		}
	case 338:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:863
		{
			yyVAL.node = binary_op.NewNotEqual(yyDollar[1].node, yyDollar[3].node)
		}
	case 339:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:864
		{
			yyVAL.node = binary_op.NewSmaller(yyDollar[1].node, yyDollar[3].node)
		}
	case 340:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:865
		{
			yyVAL.node = binary_op.NewSmallerOrEqual(yyDollar[1].node, yyDollar[3].node)
		}
	case 341:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:866
		{
			yyVAL.node = binary_op.NewGreater(yyDollar[1].node, yyDollar[3].node)
		}
	case 342:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:867
		{
			yyVAL.node = binary_op.NewGreaterOrEqual(yyDollar[1].node, yyDollar[3].node)
		}
	case 343:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:868
		{
			yyVAL.node = binary_op.NewSpaceship(yyDollar[1].node, yyDollar[3].node)
		}
	case 344:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:869
		{
			yyVAL.node = expr.NewInstanceOf(yyDollar[1].node, yyDollar[3].node)
		}
	case 345:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:870
		{
			yyVAL.node = yyDollar[2].node
		}
	case 346:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:871
		{
			yyVAL.node = yyDollar[1].node
		}
	case 347:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:872
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, yyDollar[3].node, yyDollar[5].node)
		}
	case 348:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:873
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, nil, yyDollar[4].node)
		}
	case 349:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:874
		{
			yyVAL.node = binary_op.NewCoalesce(yyDollar[1].node, yyDollar[3].node)
		}
	case 350:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:875
		{
			yyVAL.node = yyDollar[1].node
		}
	case 351:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:876
		{
			yyVAL.node = cast.NewCastInt(yyDollar[2].node)
		}
	case 352:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:877
		{
			yyVAL.node = cast.NewCastDouble(yyDollar[2].node)
		}
	case 353:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:878
		{
			yyVAL.node = cast.NewCastString(yyDollar[2].node)
		}
	case 354:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:879
		{
			yyVAL.node = cast.NewCastArray(yyDollar[2].node)
		}
	case 355:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:880
		{
			yyVAL.node = cast.NewCastObject(yyDollar[2].node)
		}
	case 356:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:881
		{
			yyVAL.node = cast.NewCastBool(yyDollar[2].node)
		}
	case 357:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:882
		{
			yyVAL.node = cast.NewCastUnset(yyDollar[2].node)
		}
	case 358:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:883
		{
			yyVAL.node = expr.NewExit(yyDollar[2].node, strings.EqualFold(yyDollar[1].token.Value, "die"))
		}
	case 359:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:884
		{
			yyVAL.node = expr.NewErrorSuppress(yyDollar[2].node)
		}
	case 360:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:885
		{
			yyVAL.node = yyDollar[1].node
		}
	case 361:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:886
		{
			yyVAL.node = expr.NewShellExec(yyDollar[2].list)
		}
	case 362:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:887
		{
			yyVAL.node = expr.NewPrint(yyDollar[2].node)
		}
	case 363:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:888
		{
			yyVAL.node = expr.NewYield(nil, nil)
		}
	case 364:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:889
		{
			yyVAL.node = expr.NewYield(nil, yyDollar[2].node)
		}
	case 365:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:890
		{
			yyVAL.node = expr.NewYield(yyDollar[2].node, yyDollar[4].node)
		}
	case 366:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:891
		{
			yyVAL.node = expr.NewYieldFrom(yyDollar[2].node)
		}
	case 367:
		yyDollar = yyS[yypt-10 : yypt+1]
		//line parser/parser.y:893
		{
			yyVAL.node = expr.NewClosure(yyDollar[4].list, yyDollar[6].list, yyDollar[7].node, yyDollar[9].list, false, yyDollar[2].boolean)
		}
	case 368:
		yyDollar = yyS[yypt-11 : yypt+1]
		//line parser/parser.y:897
		{
			yyVAL.node = expr.NewClosure(yyDollar[5].list, yyDollar[7].list, yyDollar[8].node, yyDollar[10].list, true, yyDollar[3].boolean)
		}
	case 369:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:903
		{
			yyVAL.boolean = false
		}
	case 370:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:904
		{
			yyVAL.boolean = true
		}
	case 371:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:908
		{
			yyVAL.list = []node.Node{}
		}
	case 372:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:909
		{
			yyVAL.list = yyDollar[3].list
		}
	case 373:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:913
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 374:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:914
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 375:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:918
		{
			yyVAL.node = expr.NewClusureUse(expr.NewVariable(node.NewIdentifier(yyDollar[1].token)), false)
		}
	case 376:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:919
		{
			yyVAL.node = expr.NewClusureUse(expr.NewVariable(node.NewIdentifier(yyDollar[2].token)), true)
		}
	case 377:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:923
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].list)
		}
	case 378:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:925
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].list)
		}
	case 379:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:927
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].list)
		}
	case 380:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:928
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].list)
		}
	case 381:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:932
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 382:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:933
		{
			yyVAL.node = yyDollar[1].node
		}
	case 383:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:937
		{
			yyVAL.node = yyDollar[1].node
		}
	case 384:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:938
		{
			yyVAL.node = yyDollar[1].node
		}
	case 385:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:942
		{
			yyVAL.node = nil
		}
	case 386:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:943
		{
			yyVAL.node = yyDollar[2].node
		}
	case 387:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:947
		{
			yyVAL.list = []node.Node{}
		}
	case 388:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:948
		{
			yyVAL.list = []node.Node{scalar.NewEncapsedStringPart(yyDollar[1].token)}
		}
	case 389:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:949
		{
			yyVAL.list = yyDollar[1].list
		}
	case 390:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:953
		{
			yyVAL.list = nil
		}
	case 391:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:954
		{
			yyVAL.list = yyDollar[1].list
		}
	case 392:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:958
		{
			yyVAL.node = expr.NewArray(yyDollar[1].token, yyDollar[4].token, yyDollar[3].list)
		}
	case 393:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:959
		{
			yyVAL.node = expr.NewShortArray(yyDollar[1].token, yyDollar[3].token, yyDollar[2].list)
		}
	case 394:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:960
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token)
		}
	case 395:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:964
		{
			yyVAL.node = scalar.NewLnumber(yyDollar[1].token)
		}
	case 396:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:965
		{
			yyVAL.node = scalar.NewDnumber(yyDollar[1].token)
		}
	case 397:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:966
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 398:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:967
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 399:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:968
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 400:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:969
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 401:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:970
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 402:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:971
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 403:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:972
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 404:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:973
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token)
		}
	case 405:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:975
		{
			yyVAL.node = scalar.NewString(yyDollar[2].token) /* TODO: mark as Heredoc*/
		}
	case 406:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:977
		{
			yyVAL.node = scalar.NewEncapsed(yyDollar[1].token, nil, yyDollar[2].token)
		}
	case 407:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:978
		{
			yyVAL.node = scalar.NewEncapsed(yyDollar[1].token, yyDollar[2].list, yyDollar[3].token)
		}
	case 408:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:979
		{
			yyVAL.node = scalar.NewEncapsed(yyDollar[1].token, yyDollar[2].list, yyDollar[3].token)
		}
	case 409:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:980
		{
			yyVAL.node = yyDollar[1].node
		}
	case 410:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:981
		{
			yyVAL.node = yyDollar[1].node
		}
	case 411:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:985
		{
			yyVAL.node = expr.NewConstFetch(yyDollar[1].node)
		}
	case 412:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:986
		{
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, yyDollar[3].token)
		}
	case 413:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:988
		{
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, yyDollar[3].token)
		}
	case 414:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:992
		{
			yyVAL.node = yyDollar[1].node
		}
	case 415:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:993
		{
			yyVAL.node = yyDollar[1].node
		}
	case 416:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:997
		{
			yyVAL.node = nil
		}
	case 417:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:998
		{
			yyVAL.node = yyDollar[1].node
		}
	case 418:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1002
		{
			yyVAL.node = yyDollar[1].node
		}
	case 419:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1006
		{
			yyVAL.node = yyDollar[1].node
		}
	case 420:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1007
		{
			yyVAL.node = yyDollar[2].node
		}
	case 421:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1008
		{
			yyVAL.node = yyDollar[1].node
		}
	case 422:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1012
		{
			yyVAL.node = yyDollar[1].node
		}
	case 423:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1013
		{
			yyVAL.node = yyDollar[2].node
		}
	case 424:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1014
		{
			yyVAL.node = yyDollar[1].node
		}
	case 425:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1018
		{
			yyVAL.node = yyDollar[1].node
		}
	case 426:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1019
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 427:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1020
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 428:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1021
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 429:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1023
		{
			yyVAL.node = expr.NewMethodCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].list)
		}
	case 430:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1024
		{
			yyVAL.node = yyDollar[1].node
		}
	case 431:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1028
		{
			yyVAL.node = yyDollar[1].node
		}
	case 432:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1029
		{
			yyVAL.node = yyDollar[1].node
		}
	case 433:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1030
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 434:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1034
		{
			yyVAL.node = expr.NewVariable(node.NewIdentifier(yyDollar[1].token))
		}
	case 435:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1035
		{
			yyVAL.node = expr.NewVariable(yyDollar[3].node)
		}
	case 436:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1036
		{
			yyVAL.node = expr.NewVariable(yyDollar[2].node)
		}
	case 437:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1041
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 438:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1043
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 439:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1047
		{
			yyVAL.node = yyDollar[1].node
		}
	case 440:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1048
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 441:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1049
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 442:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1050
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 443:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1052
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 444:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1054
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
		}
	case 445:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1058
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 446:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1059
		{
			yyVAL.node = yyDollar[2].node
		}
	case 447:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1060
		{
			yyVAL.node = yyDollar[1].node
		}
	case 448:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1064
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token)
		}
	case 449:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1065
		{
			yyVAL.node = yyDollar[2].node
		}
	case 450:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1066
		{
			yyVAL.node = yyDollar[1].node
		}
	case 451:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1071
		{
			if yyDollar[1].list[len(yyDollar[1].list)-1] == nil {
				yyVAL.list = yyDollar[1].list[:len(yyDollar[1].list)-1]
			} else {
				yyVAL.list = yyDollar[1].list
			}
		}
	case 452:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line parser/parser.y:1081
		{
			yyVAL.node = nil
		}
	case 453:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1082
		{
			yyVAL.node = yyDollar[1].node
		}
	case 454:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1087
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 455:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1088
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 456:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1092
		{
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, yyDollar[3].node, false)
		}
	case 457:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1093
		{
			yyVAL.node = expr.NewArrayItem(nil, yyDollar[1].node, false)
		}
	case 458:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1094
		{
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, yyDollar[4].node, true)
		}
	case 459:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1095
		{
			yyVAL.node = expr.NewArrayItem(nil, yyDollar[2].node, true)
		}
	case 460:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:1097
		{
			// TODO: Cannot use list() as standalone expression
			list := expr.NewList(yyDollar[5].list)
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, list, false)
		}
	case 461:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1103
		{
			// TODO: Cannot use list() as standalone expression
			list := expr.NewList(yyDollar[3].list)
			yyVAL.node = expr.NewArrayItem(nil, list, false)
		}
	case 462:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1111
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 463:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1112
		{
			yyVAL.list = append(yyDollar[1].list, scalar.NewEncapsedStringPart(yyDollar[2].token))
		}
	case 464:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1113
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 465:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1114
		{
			yyVAL.list = []node.Node{scalar.NewEncapsedStringPart(yyDollar[1].token), yyDollar[2].node}
		}
	case 466:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1118
		{
			yyVAL.node = expr.NewVariable(node.NewIdentifier(yyDollar[1].token))
		}
	case 467:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1119
		{
			yyVAL.node = expr.NewArrayDimFetch(expr.NewVariable(node.NewIdentifier(yyDollar[1].token)), yyDollar[3].node)
		}
	case 468:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1120
		{
			yyVAL.node = expr.NewPropertyFetch(expr.NewVariable(node.NewIdentifier(yyDollar[1].token)), node.NewIdentifier(yyDollar[3].token))
		}
	case 469:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1121
		{
			yyVAL.node = expr.NewVariable(yyDollar[2].node)
		}
	case 470:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1122
		{
			yyVAL.node = expr.NewVariable(node.NewIdentifier(yyDollar[2].token))
		}
	case 471:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line parser/parser.y:1124
		{
			yyVAL.node = expr.NewArrayDimFetch(expr.NewVariable(node.NewIdentifier(yyDollar[2].token)), yyDollar[4].node)
		}
	case 472:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1125
		{
			yyVAL.node = yyDollar[2].node
		}
	case 473:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1128
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token)
		}
	case 474:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1130
		{
			// TODO: add option to handle 64 bit integer
			if _, err := strconv.Atoi(yyDollar[1].token.Value); err == nil {
				yyVAL.node = scalar.NewLnumber(yyDollar[1].token)
			} else {
				yyVAL.node = scalar.NewString(yyDollar[1].token)
			}
		}
	case 475:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1139
		{
			// TODO: add option to handle 64 bit integer
			if _, err := strconv.Atoi(yyDollar[2].token.Value); err == nil {
				yyVAL.node = expr.NewUnaryMinus(scalar.NewLnumber(yyDollar[2].token))
			} else {
				yyDollar[2].token.Value = "-" + yyDollar[2].token.Value
				yyVAL.node = scalar.NewString(yyDollar[2].token)
			}
		}
	case 476:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1148
		{
			yyVAL.node = expr.NewVariable(node.NewIdentifier(yyDollar[1].token))
		}
	case 477:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line parser/parser.y:1152
		{
			yyVAL.node = expr.NewIsset(yyDollar[3].list)
		}
	case 478:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1153
		{
			yyVAL.node = expr.NewEmpty(yyDollar[3].node)
		}
	case 479:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1154
		{
			yyVAL.node = expr.NewInclude(yyDollar[2].node)
		}
	case 480:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1155
		{
			yyVAL.node = expr.NewIncludeOnce(yyDollar[2].node)
		}
	case 481:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line parser/parser.y:1156
		{
			yyVAL.node = expr.NewEval(yyDollar[3].node)
		}
	case 482:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1157
		{
			yyVAL.node = expr.NewRequire(yyDollar[2].node)
		}
	case 483:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line parser/parser.y:1158
		{
			yyVAL.node = expr.NewRequireOnce(yyDollar[2].node)
		}
	case 484:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1162
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 485:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line parser/parser.y:1163
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 486:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line parser/parser.y:1167
		{
			yyVAL.node = yyDollar[1].node
		}
	}
	goto yystack /* stack new state and value */
}
